//g++ -o main_double_pendulum main_double_pendulum.cc.cc `pkg-config --cflags --libs lib_3d_mec_ginac-1.1 ginac cln`
//g++ -o main_double_pendulum  main_double_pendulum.cc -I../../../include  -L../../../lib -l_3d_mec_ginac-1.1 -lginac -lcln -lgmp

#include <fstream>
#include "lib_3d_mec_ginac/lib_3d_mec_ginac.h"
#include <ginac/ginac.h>
using namespace GiNaC;

using namespace std;

void printError(char* arg){
	printf("%s\n", arg);
}	
	

int main(int argc, char *argv[]) {

if (argc!=4){
		printf("Error: The program %s takes 3 parameters: Gravity (UP/DOWN), Atomize (YES/NO) and formalism(NewEul/Lagrange)\n",argv[0]);
		exit(1);
	}
	
if (0==strcmp(argv[1],"DOWN")){
	gravity=DOWN;
	cout << "gravity=DOWN" << endl;
}
else{
	if (0==strcmp(argv[1],"UP")){
		gravity=UP;
		cout << "gravity=UP" << endl;
	}
	else
	{
		printf("Error: The program %s takes 3 parameters: Gravity (UP/DOWN), Atomize (YES/NO) and formalism(NewEul/Lagrange)\n",argv[0]);
		exit(1);
	}
}

if (0==strcmp(argv[2],"YES")){
		atomization=YES;
		cout << "atomization=YES" << endl;
}
else{
	if (0==strcmp(argv[2],"NO")){
		atomization=NO;
		cout << "atomization=NO" << endl;
	}
	else
	{
		printf("Error: The program %s takes 3 parameters: Gravity (UP/DOWN), Atomize (YES/NO) and formalism(NewEul/Lagrange)\n",argv[0]);
		exit(1);
	}
}

#define NewEul 0
#define Lagrange 1
int formalism;

if (0==strcmp(argv[3],"NewEul")){
		formalism=NewEul;
		cout << "Formalism=NewEul" << endl;
}
else{
	if (0==strcmp(argv[3],"Lagrange")){
		formalism=Lagrange;
		cout << "Formalism=Lagrange" << endl;
	}
	else
	{
		printf("Error: The program %s takes 3 parameters: Gravity (UP/DOWN), Atomize (YES/NO) and formalism(NewEul/Lagrange)\n",argv[0]);
		exit(1);
	}
}


//System definition
	System sys(&printError);

//Coordinate definition
	symbol theta1 = *sys.new_Coordinate("theta1",1.0,0,0);
	symbol theta2 = *sys.new_Coordinate("theta2", 3.1416/2.0,0,0);

//Kinematical parameter definition	
	symbol l1 = *sys.new_Parameter("l1",1);
	symbol l2 = *sys.new_Parameter("l2",1);

//Define Base
 	sys.new_Base("BPendulum1","xyz",1,0,0, theta1);
	sys.new_Base("BPendulum2","BPendulum1",1,0,0, theta2);

//Define Vectors
	
	Vector3D OG1 = *sys.new_Vector3D("OG1",0,0,l1/2,"BPendulum1");
	
	Vector3D OO2 = *sys.new_Vector3D("OO2",0,0,l1,"BPendulum1");
	
	Vector3D O2G2 = *sys.new_Vector3D("O2G2",0,0,l2/2,"BPendulum2");

//Define Points
	
	Point * G1 = sys.new_Point("G1","O",&OG1);
	Point * O2 = sys.new_Point("O2","O",&OO2);
	Point * G2 = sys.new_Point("G2","O2",&O2G2);

//Define Frames
	Frame * Pendulum1 = sys.new_Frame("Pendulum1","G1","BPendulum1");
	Frame * Pendulum2 = sys.new_Frame("Pendulum2","G2","BPendulum2");


//Dynamical Parameter Definition
	symbol  mpendulum1 = *sys.new_Parameter("mpendulum1",1);
	symbol  mpendulum2 = *sys.new_Parameter("mpendulum2",1);
	symbol  g = *sys.new_Parameter("g",10);
	symbol  I1 = *sys.new_Parameter("I1",1);
	symbol  I2 = *sys.new_Parameter("I2",1);

	Tensor3D  Ipendulum1G1 = *sys.new_Tensor3D("Ipendulum1G1",
					(ex)I1,(ex)0,(ex)0,(ex)0,(ex)0,(ex)0,(ex)0,(ex)0,(ex)0,"BPendulum1");
	Tensor3D  Ipendulum2G2 = *sys.new_Tensor3D("Ipendulum2G2",
					(ex)I2,(ex)0,(ex)0,(ex)0,(ex)0,(ex)0,(ex)0,(ex)0,(ex)0,"BPendulum1");
//Joint Unknown Definition
	symbol  Fgp12 = *sys.new_Joint_Unknown("Fgp12");
	symbol  Fgp13 = *sys.new_Joint_Unknown("Fgp13");
	
	symbol  Fp1p22 = *sys.new_Joint_Unknown("Fp1p22");
	symbol  Fp1p23 = *sys.new_Joint_Unknown("Fp1p23");
	

//Joint Torsor Definition
	Vector3D F_GP1 = *sys.new_Vector3D("F_GB",0,Fgp12,Fgp13,"BPendulum1");
	
	Vector3D M_GP1_O = *sys.new_Vector3D("M_GB_A",0,0,0,"BPendulum1");
	
	Vector3D F_P1P2 = *sys.new_Vector3D("F_BP",0,Fp1p22,Fp1p23,"BPendulum2");
	
	Vector3D M_P1P2_O2 = *sys.new_Vector3D("M_BP_Gb",0,0,0,"BPendulum2");
	
//Constitutive Forces and moments Definition
	Vector3D Pendulum1_gravity = *sys.new_Vector3D("mpendulum1*g"
						,0,0,-mpendulum1 * g,"xyz");
	
	Vector3D Pendulum2_gravity = *sys.new_Vector3D("mpendulum2*g"
						,0,0,-mpendulum2 * g,"xyz");
	
//Define Velocity and Acceleration of Point Gb
	//Vector3D OG1 = sys.Position_Vector("O","G1");
        Vector3D VabsG1 = sys.Dt(OG1,"abs");	
	Vector3D AabsG1 = sys.Dt(VabsG1,"abs");
	
//Define Acceleration of Point Gp
	Vector3D OG2 = sys.Position_Vector("O","G2");
	Vector3D VabsG2 = sys.Dt(OG2,"abs");
	Vector3D AabsG2 = sys.Dt(VabsG2,"abs");	

//Angular moment and Inertia Moment of pendulum
	Vector3D OmegaPendulum1 = sys.Angular_Velocity("xyz","BPendulum1");
	
	Vector3D H_G1 = Ipendulum1G1 * OmegaPendulum1;
	
	Vector3D Iner_Moment_Pendulum1_G1 = -sys.Dt(H_G1,"abs");

	Vector3D OmegaPendulum2 = sys.Angular_Velocity("xyz","BPendulum2");

	Vector3D H_G2 = Ipendulum2G2 * OmegaPendulum2;
	
	Vector3D Iner_Moment_Pendulum2_G2 = -sys.Dt(H_G2,"abs");

//Dynamic Equations Newton Euler

	if (formalism==NewEul){

	Matrix equation_1_3 = -mpendulum1 * AabsG1 + Pendulum1_gravity + F_GP1 - F_P1P2;
	Matrix equation_2_3 = Iner_Moment_Pendulum1_G1 + ((sys.Position_Vector("G1","O") ^ F_GP1) +  M_GP1_O)- ((sys.Position_Vector("G1","O2") ^ F_P1P2) + M_P1P2_O2);
	Matrix equation_3_3 = -mpendulum2 * AabsG2 + Pendulum2_gravity + F_P1P2;
	Matrix equation_4_3 = Iner_Moment_Pendulum2_G2 + ((sys.Position_Vector("G2","O2") ^ F_P1P2) + M_P1P2_O2);

	Matrix Dynamic_Equations(6,1,&equation_1_3(1,0),&equation_1_3(2,0),&equation_2_3(0,0),&equation_3_3(1,0),&equation_3_3(2,0),&equation_4_3(0,0));

	Matrix q = sys.Coordinates();
	Matrix dq = sys.Velocities();
	Matrix ddq = sys.Accelerations();
	Matrix epsilon = sys.Joint_Unknowns();
	
	Matrix M=sys.jacobian(Dynamic_Equations.transpose(),ddq);
	Matrix V=sys.jacobian(Dynamic_Equations.transpose(),epsilon);
	Matrix Q=-(Dynamic_Equations-(M*ddq+V*epsilon));
	Matrix MV(1,2,&M,&V);
	
//Output
	cout << q << endl;
	cout << dq << endl;
	cout << ddq << endl;
	cout << epsilon << endl;
	cout << Dynamic_Equations << endl;
	cout << MV << endl;
	cout << Q << endl;

	for (int i=0; (i < atoms.size())&& (i >0); ++i) { cout << i << " " << atoms.size()-1 << endl;cout << atoms[i] << " = " << atom_expressions[i] << endl;}

//Export C code for Direct Simulation

        sys.export_var_def_C();//->"var_def.c"
        sys.export_var_def_H();//->"var_def.c"
        sys.export_var_init_C();//->"var_init.c"
	sys.export_gen_coord_vect_def_H();//->"gen_coord_vect_def.c"
        sys.export_gen_coord_vect_init_C();//->"gen_coord_vect_init.c"
        sys.export_gen_vel_vect_init_H();//->"gen_vel_vect_init.c"
        sys.export_gen_vel_vect_init_C();//->"gen_vel_vect_init.c"
        sys.export_gen_accel_vect_init_H();//->"gen_accel_vect_init.c"	
        sys.export_gen_accel_vect_init_C();//->"gen_accel_vect_init.c"
        sys.export_unknowns_vect_init_H();//->"unknowns_vect_init.c"
        sys.export_unknowns_vect_init_C();//->"unknowns_vect_init.c"

	lst used_atom_list, new_atom_list_Q, new_atom_list_MV;
	new_atom_list_Q=atom_list(Q, used_atom_list);
	new_atom_list_MV=atom_list(MV, used_atom_list);
	sys.export_atom_def_C(used_atom_list);//->"atoms_def.c" and "atoms_def.h"	

//	Repeated evaluation of atoms avoided if calling sequence is used.
        sys.export_Column_Matrix_C("Q", "Phi", Q, new_atom_list_Q);//->"Q.c" and "Q.h" with Q(Phi);
        sys.export_Matrix_C("MV", "J", MV, new_atom_list_MV);//->"MV.c" and "MV.h" with MV(J);

//	Repeated evaluation of atoms in sequential function evaluation not avoided
//	sys.export_Column_Matrix_C("Q", "Phi", Q, used_atom_list);//->"Q.c" and "Q.h" with Q(Phi);
//      sys.export_Matrix_C("MV", "J", MV, used_atom_list);//->"MV.c" and "MV.h" with MV(J);

        sys.export_write_state_file_header_C();
	sys.export_write_state_file_C();

//Compile exported code

	system("gcc -DHAVE_INLINE=1 -DGSL_RANGE_CHECK_OFF=1 -o main_NewEul  main_NewEul.c var_def.c var_init.c gen_coord_vect_def.c gen_coord_vect_init.c gen_vel_vect_def.c gen_vel_vect_init.c gen_accel_vect_def.c gen_accel_vect_init.c unknowns_vect_def.c unknowns_vect_init.c Q.c MV.c  write_state_file.c write_state_file_header.c qr_nm.c newton_raphson_qr.c printing.c  -lgsl -lgslcblas -lm -lcln");

//Execute compiled program
	system("./main_NewEul 10 0.001");

//Show graphics of simulation
	system("gnuplot plot_NewEul.gnuplot");

	}

	if (formalism==Lagrange){

//Dynamic Equations Lagrange
	symbol *dtheta1=sys.get_Velocity("dtheta1");
	ex equation_1 = (-mpendulum1 * AabsG1 + Pendulum1_gravity)*sys.diff(VabsG1,*dtheta1) + Iner_Moment_Pendulum1_G1 * sys.diff(OmegaPendulum1,*dtheta1) + (-mpendulum2 * AabsG2 + Pendulum2_gravity) * sys.diff(VabsG2,*dtheta1) + Iner_Moment_Pendulum2_G2 * sys.diff(OmegaPendulum2,*dtheta1);
	symbol *dtheta2=sys.get_Velocity("dtheta2");
	ex equation_2 = (-mpendulum1 * AabsG1 + Pendulum1_gravity)*sys.diff(VabsG1,*dtheta2) + Iner_Moment_Pendulum1_G1 * sys.diff(OmegaPendulum1,*dtheta2) + (-mpendulum2 * AabsG2 + Pendulum2_gravity) * sys.diff(VabsG2,*dtheta2) + Iner_Moment_Pendulum2_G2 * sys.diff(OmegaPendulum2,*dtheta2);

	Matrix Dynamic_Equations(2,1,&equation_1,&equation_2);

	Matrix q = sys.Coordinates();
	Matrix dq = sys.Velocities();
	Matrix ddq = sys.Accelerations();
	Matrix epsilon = sys.Joint_Unknowns();
	
	Matrix M=sys.jacobian(Dynamic_Equations.transpose(),ddq);
	Matrix Q=-(Dynamic_Equations-(M*ddq));
	symbol *ddtheta1=sys.get_Acceleration("ddtheta1");
	symbol *ddtheta2=sys.get_Acceleration("ddtheta2");

	Q(0,0)=Q(0,0).subs((*ddtheta1)==0);
	Q(0,0)=Q(0,0).subs((*ddtheta2)==0);
	Q(1,0)=Q(1,0).subs((*ddtheta1)==0);
	Q(1,0)=Q(1,0).subs((*ddtheta2)==0);

//Output
	cout << q << endl;
	cout << dq << endl;
	cout << ddq << endl;
	cout << epsilon << endl;
	cout << Dynamic_Equations << endl;
	cout << M << endl;
	cout << Q << endl;

	for (int i=0; (i < atoms.size())&& (i >0); ++i) {
		cout << i << " " << atoms.size()-1 << endl;
		cout << atoms[i] << " = " << atom_expressions[i] << endl;
	}


//Export C code for Direct Simulation

        sys.export_var_def_C();//->"var_def.c"
        sys.export_var_def_H();//->"var_def.c"
        sys.export_var_init_C();//->"var_init.c"
	sys.export_gen_coord_vect_def_H();//->"gen_coord_vect_def.c"
        sys.export_gen_coord_vect_init_C();//->"gen_coord_vect_init.c"
        sys.export_gen_vel_vect_init_H();//->"gen_vel_vect_init.c"
        sys.export_gen_vel_vect_init_C();//->"gen_vel_vect_init.c"
        sys.export_gen_accel_vect_init_H();//->"gen_accel_vect_init.c"	
        sys.export_gen_accel_vect_init_C();//->"gen_accel_vect_init.c"
        sys.export_unknowns_vect_init_H();//->"unknowns_vect_init.c"
        sys.export_unknowns_vect_init_C();//->"unknowns_vect_init.c"

	lst used_atom_list, new_atom_list_Q, new_atom_list_M;
	new_atom_list_Q=atom_list(Q, used_atom_list);
	new_atom_list_M=atom_list(M, used_atom_list);
	sys.export_atom_def_C(used_atom_list);//->"atoms_def.c" and "atoms_def.h"	

//	Repeated evaluation of atoms avoided if calling sequence is used.
        sys.export_Column_Matrix_C("Q", "Phi", Q, new_atom_list_Q);//->"Q.c" and "Q.h" with Q(Phi);
        sys.export_Matrix_C("M", "J", M, new_atom_list_M);//->"MV.c" and "MV.h" with MV(J);

//	Repeated evaluation of atoms in sequential function evaluation not avoided
//	sys.export_Column_Matrix_C("Q", "Phi", Q, used_atom_list);//->"Q.c" and "Q.h" with Q(Phi);
//      sys.export_Matrix_C("M", "J", M, used_atom_list);//->"MV.c" and "MV.h" with MV(J);

        sys.export_write_state_file_header_C();
	sys.export_write_state_file_C();

//Compile exported code

	system("gcc -DHAVE_INLINE=1 -DGSL_RANGE_CHECK_OFF=1 -o main_Lagrange  main_Lagrange.c var_def.c var_init.c gen_coord_vect_def.c gen_coord_vect_init.c gen_vel_vect_def.c gen_vel_vect_init.c gen_accel_vect_def.c gen_accel_vect_init.c unknowns_vect_def.c unknowns_vect_init.c Q.c M.c  write_state_file.c write_state_file_header.c qr_nm.c newton_raphson_qr.c printing.c  -lgsl -lgslcblas -lm -lcln");

//Execute compiled program
	system("./main_Lagrange 10 0.001");

//Show graphics of simulation
	system("gnuplot plot_Lagrange.gnuplot");

	}

return 0;

}


