//  Fast, change -march to suti your needs (more /proc/cpuinfo) 
// gcc -O3  -DHAVE_INLINE=1 -DGSL_RANGE_CHECK_OFF=1 -o main  main.c var_def.c var_init.c gen_coord_vect_def.c gen_coord_vect_init.c gen_vel_vect_def.c gen_vel_vect_init.c gen_accel_vect_def.c gen_accel_vect_init.c unknowns_vect_def.c unknowns_vect_init.c Q.c MV.c  write_state_file.c write_state_file_header.c qr_nm.c newton_raphson_qr.c printing.c  -lgsl -lgslcblas -lm -lcln -march=i686 -fomit-frame-pointer

//  Probably as fast as above and less error prone, change -march to suti your needs (more /proc/cpuinfo)
// gcc -O3  -DHAVE_INLINE=1 -DGSL_RANGE_CHECK_OFF=1 -o main  main.c var_def.c var_init.c gen_coord_vect_def.c gen_coord_vect_init.c gen_vel_vect_def.c gen_vel_vect_init.c gen_accel_vect_def.c gen_accel_vect_init.c unknowns_vect_def.c unknowns_vect_init.c Q.c MV.c  write_state_file.c write_state_file_header.c qr_nm.c newton_raphson_qr.c printing.c  -lgsl -lgslcblas -lm -lcln -march=i686 -fomit-frame-pointer

//  Debbuging // gcc -g  -DHAVE_INLINE=1 -DGSL_RANGE_CHECK_OFF=1 -o main3-i686  main.c var_def.c var_init.c gen_coord_vect_def.c gen_coord_vect_init.c gen_vel_vect_def.c gen_vel_vect_init.c gen_accel_vect_def.c gen_accel_vect_init.c unknowns_vect_def.c unknowns_vect_init.c Q.c MV.c  write_state_file.c write_state_file_header.c qr_nm.c newton_raphson_qr.c printing.c  -lgsl -lgslcblas -lm -lcln 

#include <stdio.h>
#include <math.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
#include "var_def.h"

#include "gen_coord_vect_def.h"
#include "gen_vel_vect_def.h"
#include "gen_accel_vect_def.h"
#include "unknowns_vect_def.h"
#include "Q.h"
#include "MV.h"
#include "write_state_file_header.h"
#include "write_state_file.h"

#include "printing.h"

#include "atom_def.c"


int main (int argc, char *argv[]) {

	if (argc!=3){
		printf("Error: The program %s takes two parameters: %s integration_time delta_t\n",argv[0],argv[0]);
		exit(1);
	}
	double integration_time=strtod(argv[1], NULL);
	double delta_t= strtod(argv[2], NULL);
	long int k,steps;
	
	printf("integration_time %g delta_t %g\n", integration_time, delta_t);

//initialize variables
	var_init();

//initialize generalized coordinate, velocity and acceleration vectors
	gen_coord_vect_init();
	gen_vel_vect_init();
	gen_accel_vect_init();

//initialize joint force unknowns
	unknowns_vect_init();


//Solve dynamic problem
	printf("q=\n");
	vector_printf(q);
	printf("dq=\n");
	vector_printf(dq);


	// Position Correction (obtained q does not have to satisfy geometric constraints)
	//	newton_raphson_qr (MV, Q, q, dq);
	
	// Velocity Correction (obtained q does not have to satisfy velocity constraints)
	//	Qv=vel_indep_term(Qv, q, dq);
	//	Jv=vel_jacobian(Jv,q, dq);
	//	gsl_permutation * pv = gsl_permutation_alloc (n_gen_vel);
	//	qr_nm(Jv, Qv, pv, dq);


	//Phi=Q(Phi);   printf("Qd=\n"); vector_printf(Qd);
	Phi=Q();   printf("Phi=\n"); vector_printf(Phi);
	
	//J=MV(J);   printf("Jd=\n");matrix_printf(Jd);
	J=MV();   printf("J=\n");matrix_printf(J);

	gsl_permutation * pd = gsl_permutation_alloc (n_gen_accel+n_unknowns); printf("pd=\n");permutation_printf(pd);
	gsl_vector * ddqunknowns= gsl_vector_alloc (n_gen_accel+n_unknowns);

	size_t i;

	for (i=0; i<n_gen_accel; i++)
		gsl_vector_set (ddqunknowns, i, gsl_vector_get(ddq,i));

	for (i=0; i<n_unknowns; i++)
    		gsl_vector_set (ddqunknowns, n_gen_accel+i, gsl_vector_get(unknowns,i));

	printf("ddqunknowns=\n"); vector_printf(ddqunknowns);
	qr_nm(J, Phi, pd, ddqunknowns);
	
	printf("pd=\n");permutation_printf(pd);
	printf("ddqunknowns=\n"); vector_printf(ddqunknowns);

	for (i=0; i<n_gen_accel; i++)
		gsl_vector_set (ddq, i, gsl_vector_get(ddqunknowns,i));
	for (i=0; i<n_unknowns; i++)
		gsl_vector_set (unknowns, i, gsl_vector_get(ddqunknowns,n_gen_accel+i));

	printf("ddq=\n"); vector_printf(ddq);
	printf("lambda1=\n"); vector_printf(unknowns);
	printf("Going to Integrate\n");

//integration euler modified

	steps=ceil(integration_time/delta_t);

	printf("integration_time=%g,  deta_t=%g, steps=%d\n",integration_time,delta_t,steps);

	gsl_vector * aux_vector = gsl_vector_alloc (q->size);

	FILE * state_file = fopen("state.dat", "w");

	write_state_file_header(state_file);

	write_state_file(state_file);

	printf("t = %g, %d of %d\n",t,k,steps);

	for (k=1; k<=steps; k++){
	// Position Actualization
		// q=q+dq*delta_t+1/2*ddq*delta_t*delta_t;

		gsl_vector_memcpy (aux_vector, ddq);
		gsl_vector_scale(aux_vector, 0.5*delta_t);
		gsl_vector_add(aux_vector,dq);
		gsl_vector_scale(aux_vector,delta_t);
		gsl_vector_add(q,aux_vector);

	// Velocity Actualization
		// dq=dq+ddq*delta_t;

		gsl_vector_memcpy (aux_vector, ddq);
		gsl_vector_scale(aux_vector, delta_t);
		gsl_vector_add(dq,aux_vector);

	// Time actualization
		t=t+delta_t;

	
	// Position Correction (obtained q does not have to satisfy geometric constraints)
	//	newton_raphson_qr (MV, Q, q, dq);
	
	// Velocity Correction (obtained q does not have to satisfy velocity constraints)
	//	Qv=vel_indep_term(Qv, q, dq);
	//	Jv=vel_jacobian(Jv,q, dq);
	//	gsl_permutation_init (pv);
	//	qr_nm(Jv, Qv, pv, dq);
	

	// Dynamic problem. Unknowns are generalized accelerations ddq and joint forces unknowns
		//Qd=Q(Qd, q, dq);
		Phi=Q();
  
  		//Jd=MV(Jd,q, dq);
		J=MV();
  		gsl_permutation_init (pd);

  		for (i=0; i<n_gen_accel; i++)
			gsl_vector_set (ddqunknowns, i, gsl_vector_get(ddq,i));
		for (i=0; i<n_unknowns; i++)
    			gsl_vector_set (ddqunknowns, n_gen_accel+i, gsl_vector_get(unknowns,i));

		qr_nm(J, Phi, pd, ddqunknowns);
		

  		for (i=0; i<n_gen_accel; i++)
			gsl_vector_set (ddq, i, gsl_vector_get(ddqunknowns,i));
		for (i=0; i<n_unknowns; i++)
			gsl_vector_set (unknowns, i, gsl_vector_get(ddqunknowns,n_gen_accel+i));
	// Note at this point, time, state (q, dq), generalized accelerations (ddq) and joint action unknowns (unknowns) are consistent except for numerical errors.

		write_state_file(state_file);

		//printf("t = %g, %d of %d\n",t,k,steps);
	}

	fclose(state_file);

	gsl_vector_free(aux_vector);
	gsl_vector_free(ddqunknowns);
  
	gsl_vector_free(q);
	gsl_vector_free(dq);
	gsl_vector_free(ddq);
	if(unknowns!=NULL)
 	gsl_vector_free(unknowns);
	gsl_vector_free(Phi);
	gsl_matrix_free(J);
	gsl_permutation_free(pd);
	//	gsl_permutation_free(pv);
	return 0;
}
