#include "Globals.h"
#include "System.h"
#include <sstream>

using GiNaC::ex;
using GiNaC::symbol;
using GiNaC::lst;
using GiNaC::is_a;
using GiNaC::wild;
using GiNaC::mul;
using GiNaC::subs_options;
using std::string;
using std::vector;
using std::ostringstream;


/* 
This function 
*/
string itoa( const long int& x ) {
	ostringstream o ;
	if ( !( o << x ) ) return "ERROR" ;
	return o.str() ;
}


/*
This function push back in exclude_atoms vector, this atom 
*/
void exclude_atom( symbol atom ){
	exclude_atoms.push_back ( atom );
}


/*
This function push back in exclude_atoms, all the coordinates 
*/
void exclude_Coordinates_atoms ( System * system ){
	vector < symbol * > coord = system-> get_Coordinates();
	for ( int i = 0  ; i < coord.size () ; i++ )
		exclude_atoms.push_back ( * coord[i] );
}
/*
This function push back in exclude_atoms vector, all the velocities
*/
void exclude_Velocities_atoms ( System * system ){
	vector < symbol * > veloc = system->get_Velocities();
	for ( int i = 0 ; i< veloc.size() ; i++ )
		exclude_atoms.push_back ( * veloc[i] );
}

/*
This function push back in exclude_atoms vector, all the accelerations
*/
void exclude_Accelerations_atoms( System * system ){
	vector < symbol * > accel = system-> get_Accelerations ();
	for ( int i = 0 ; i < accel.size () ; i++ )
		exclude_atoms.push_back ( * accel[i] );
}

/*
This function push back in exclude_atoms vector, this atom 
*/
void exclude_Unknowns_atoms( System * system ){
	vector < symbol * > unkno = system-> get_Unknowns();
	for ( int i = 0 ; i < unkno.size () ; i++ )
		exclude_atoms.push_back ( * unkno[i] );
}


/*
This function atomize an expression
*/
ex atomize_ex( ex expression ){

	if ( atomization == NO )
		return expression;
	if ( is_a < symbol > ( expression ) )
		return expression;
	if ( is_a < numeric > ( expression ) )
		return expression; 

	//The expresion is unatomized for simplify expressions
	expression = unatomize_ex( expression );

	if ( is_a < symbol > ( expression ) )
		return expression;
	if ( is_a < numeric > ( expression ) )
		return expression;

	//Now the expression is atomized other time
	for ( int i = 0 ; i < atom_expressions.size() ; i++ ){
		expression = expression.subs ( atom_expressions[i] + wild( 1 ) == atoms[i] + wild( 1 ) , subs_options::algebraic );
		expression = expression.subs ( atom_expressions[i] == atoms[i] ,subs_options::algebraic );
		expression = expression.subs ( -atom_expressions[i] + wild( 1 ) == -atoms[i] + wild( 1 ) , subs_options::algebraic );
		expression = expression.subs ( -atom_expressions[i] == -atoms[i] , subs_options::algebraic );
    	}

   	bool excluding_atoms = false;
   	ex sub_expression = expression;
   	for ( int i = 0 ; i < exclude_atoms .size() ; i++ )
    		if ( sub_expression.has( exclude_atoms[i] ) == 1 ) {
        		sub_expression = sub_expression.subs( exclude_atoms[i] == 0 , subs_options::algebraic ) ;
        		excluding_atoms = true ;
     		}
   	
	if ( excluding_atoms ) {
   		ex sub_expression2 = expression;
   		expression = atomize_ex( sub_expression );
    		for ( int i = 0 ; i<exclude_atoms.size() ; i++ )
    			if ( sub_expression2.has( exclude_atoms[i] ) == 1 ) {
			        sub_expression = diff( sub_expression2 , exclude_atoms[i] );
			        sub_expression = atomize_ex( sub_expression );
			        expression = expression + sub_expression  * exclude_atoms[i];
			}
     	return expression;
    	}

   	symbol atom ;
   	if ( expression != 0 ) {
    		if ( ( is_a < symbol > ( expression ) == 0 )and( is_a < numeric > ( expression ) == 0 ) ){
    			if (  not(  ( is_a < mul > ( expression ) == 1 ) and ( expression.nops() == 2 ) and ( is_a < symbol > ( expression.op( 0 ) ) == 1 ) and ( is_a < numeric > ( expression.op( 1 ) ) == 1 )  )  ) {
  
		      		atom = symbol ( "atom" + itoa ( atom_expressions.size() ) ) ;  
      				ex atom_expression = expression ;

		      		atoms.push_back ( atom ) ;
      				atom_expressions.push_back ( atom_expression ) ;

		      		expression = expression.subs ( atom_expression == atom,subs_options::algebraic ) ;
				expression = expression.subs ( -atom_expression == atom,subs_options::algebraic ) ;
     			}
     		}
    	}
    	return expression ;
}


/*
This function unatomize an expression
*/
ex unatomize_ex( ex expression ) {
	if ( atomization == NO )
		return expression;
	for ( int i = atom_expressions.size () - 1 ; i >= 0 ; i-- )
    		expression = expression.subs( atoms[i] == atom_expressions[i] , subs_options::algebraic );
	return expression;
}


/*
This function produces the list of atoms a expression, return is atoms in expression not contained in initial lst list, lst list in output contains the initial atoms in list and the previous atoms.
*/
lst atom_list_ex( ex expression , lst & list ){

	lst aux_list , new_atom_list;
	if ( atomization == NO )
		return new_atom_list;

	for ( int i = atoms.size () - 1 ; i >= 0 ; i-- )
		if ( expression.has ( atoms[i] ) == 1 ) {
			aux_list.prepend ( atoms[i] );
    			expression = expression.subs( atoms[i] == atom_expressions[i] , subs_options::algebraic ) ;	
		}

	for ( lst::const_iterator i = aux_list.begin () ; i != aux_list.end () ; ++i )	
		if ( list.has( * i ) == 0 ){
			list.append ( * i );
			new_atom_list.append ( * i );
		}

return new_atom_list;
}


/*
This function produces the list of expressions corresponding to the provided list of atoms.
*/
void atom_expression_list( lst & list , lst & expression_list ) {
	expression_list.remove_all ();
	if ( atomization == NO )
		return;
	int j;

	for ( int i = 0 ; i < list.nops () ; ++i ) {
		j = 0;
		while ( list.op ( i ) != atoms[j] )
			++j ;
		expression_list.append ( atom_expressions[j] ) ;	
	}
}

