#ifndef GLOBALS_H
#define GLOBALS_H

#include "Matrix.h"

using GiNaC::ex;
using GiNaC::lst;
using GiNaC::symbol;
using std::string;
using std::vector; 

#define DOWN 0
#define UP 1
#define NO 0
#define YES 1

extern int atomization;//0-> work without atomization; 1-> work with atomization
extern int gravity;//0-> work with down gravity; 1-> work with up gravity
extern void  (  * outError ) ( char *  );
void printError ( char * args );


/*
Atomize/Unatomize 
*/

extern vector < symbol >  atoms;
extern vector < ex >  atom_expressions;
extern vector < symbol >  exclude_atoms;

ex atomize_ex ( ex e );
ex unatomize_ex ( ex e );
lst atom_list_ex ( ex  , lst & );
void atom_expression_list ( lst & , lst & );

void exclude_atom ( symbol symbolA );
void exclude_Coordinates_atoms ( System * system );
void exclude_Velocities_atoms ( System * system );
void exclude_Accelerations_atoms ( System * system );
void exclude_Unknowns_atoms ( System * system );



/*
This function unatomize a Matrix
 */
template < class T >
T unatomize ( T m ){
	if  ( atomization == NO )
		return m;
	for  ( int i = 0 ; i < m.rows () ; i++ )
		for  ( int j = 0; j < m.cols () ; j++ )
			m ( i , j ) = unatomize_ex ( m ( i , j ) );
	return m;
}


/*
This function atomize a Matrix
*/
template < class T >
T atomize ( T m ){
	if  ( atomization == NO )
		return m;
	for  ( int i = 0; i < m.rows () ; i++ )
		for  ( int j = 0 ; j < m.cols () ; j++ )
			m ( i , j ) = atomize_ex ( m ( i ,j ) );
	return m;
}


/*
This function atom_list a Matrix
*/
template < class T >
lst atom_list ( T m , lst & list ){
	lst new_atom_list , aux_new_atom_list;
	if  ( atomization == NO )
		return new_atom_list;
	for  ( int i = 0 ; i < m.rows () ; i++ )
		for  ( int j = 0 ; j < m.cols () ; j++ ){
			aux_new_atom_list = atom_list_ex ( m ( i , j ) , list );
			for  ( lst::const_iterator k = aux_new_atom_list.begin () ; k != aux_new_atom_list.end () ; ++k )	
				new_atom_list.append (  * k );
			aux_new_atom_list.remove_all ();
		}
	return new_atom_list;
}


#endif
