#include "Globals.h"
#include "Matrix.h"
#include "System.h"

using GiNaC::ex;
using GiNaC::matrix;
using GiNaC::lst;
using GiNaC::relational;
using GiNaC::symbol;
using std::string;
using std::ostream;
using std::exception;

int atomization;

/* 
Method that initialize the Matrix atributes
*/
void Matrix::init ( string name , matrix mat ) {
	this-> name = name;
	this-> mat = mat;
	this-> last_row = 0;
	this-> last_col = 0;
	this-> system = NULL;
	* this = atomize ( * this ); 
}


/* 
Constructor with name and one matrix
*/
Matrix::Matrix ( string name , Matrix mat ) {
	init ( name , mat.get_matrix () ); 
}


/* 
Constructor with one matrix
*/
Matrix::Matrix ( matrix mat ) {
	init ( "" , mat ); 
}


/* 
Constructor with name and number of rows and cols initilized by 0'name
*/
Matrix::Matrix ( string name , long rows , long cols ) {
	init ( name , matrix ( rows , cols ) ); 
}


/* 
Constructor with number of rows and cols initilized by 0'name
*/
Matrix::Matrix ( long rows , long cols ) { 
	init ( "" , matrix ( rows , cols ) ); 
}


/* 
Constructor with name , number of rows and cols and the values of this
*/
Matrix::Matrix ( string name , long rows , long cols , ex * first , ... ) {
	try{
		int j = 1;

		this-> name = name;
		matrix aux ( rows , cols ); 
		aux ( 0 , 0 ) = * first;
		/* Classic form of catch a elements from a function with a undefined number of parameters*/
		va_list marcador;
		va_start ( marcador , first ); 
		for ( int i = 0 ; i < aux.rows () ; i++ ) {
			while ( j < aux.cols () ) {
				ex * exClase = va_arg ( marcador , ex * ); 
				aux ( i , j ) = * exClase;
				j++;		
			}
			j = 0;
		}
		va_end ( marcador ); 
		mat = aux;
		system = NULL;
		* this = atomize ( * this ); 
	}catch ( exception & p ) {
  		outError ( "ERR - Unspected error building Matrix" ); 
	}
}


/* 
Constructor with number of rows and cols and the values of this
*/
Matrix::Matrix ( long rows , long cols , ex * first , ... ) {
	try{
		int j = 1;

		name = "";
		matrix aux ( rows , cols ); 
		aux ( 0 , 0 ) = * first;
		//Classic form of catch a elements from a function with a undefined number of parameters
		va_list marcador;
		va_start ( marcador , first ); 
		for ( int i = 0 ; i < aux.rows () ; i++ ) {
			while ( j < aux.cols () ) {
				ex * exClase = va_arg ( marcador , ex * ); 
				aux ( i , j ) = * exClase;
				j++;		
			}
			j = 0;
		}
		va_end ( marcador ); 
		mat = aux;
		system = NULL;
		* this = atomize ( * this ); 
	}catch ( exception & p ) {
  		outError ( "ERR - Unspected error building Matrix" ); 
	}
}


/* 
Constructor with number of Matrix in rows and cols for compose a new Matrix with this Matrix
*/
Matrix::Matrix ( long rows , long cols , Matrix * first , ... ) {
	try{
		int j = 1;
		name = "";
		vector < Matrix * > auxMatrixs; 
		/* Introduce all the Matriz in the auxMatrixs vector*/
		/* Classic form of catch a elements from a function with a undefined number of parameters*/
		auxMatrixs.push_back ( first ); 
		va_list marcador;
		va_start ( marcador , first ); 
		for ( int i = 0 ; i < rows ; i++ ) {
			while ( j < cols ) {
				auxMatrixs.push_back ( va_arg ( marcador , Matrix * ) ); 
				j++;		
			}
			j = 0;
		}
		va_end ( marcador ); 

		//Calculate the number of cols of the final Matrix using auxMatrix
		int i = 0;
		long numCols = 0;
		
		while ( i<cols ) {
			numCols = numCols + auxMatrixs[i]-> cols () ; 
			i++;
		}

		//Calculate the number of rows of the final Matrix using auxMatrix
		long numRows = 0;
		i = 0;
		while ( i < auxMatrixs.size () ) {
			numRows = numRows + auxMatrixs[i]->rows () ; 
			i += cols;
		}

		/* 
		Comprove that all the Matrix are compatible in number of rows and cols
		The sum of the cols of the Matrixs in the same row should be have the same num of cols
		This num of cols is numCols
		 */
		long compNumCols = 0;
		int k = 0;
		int r = cols - 1;
		while ( k < auxMatrixs.size () ) {
			compNumCols = compNumCols + auxMatrixs[k]-> cols () ; 
			if ( compNumCols > numCols ) throw 1;
			if ( k == ( r ) ) {
				if ( compNumCols != numCols ) throw 1;
				compNumCols = 0;
				r += cols;
			}
			k++;
		}
		
		//Also should be perform that all the Matrix of the same row , have the same number of rows;
		long compNumRows = 0;
		long compNumRows2 = 0;
		k = 0;
		while ( k < auxMatrixs.size () ) {
			compNumRows = auxMatrixs[k]-> rows () ; 
			//Perform that the number of rows is the same of before
			if ( ( compNumRows != compNumRows2 ) and ( compNumRows2 != 0 ) ) throw 1;
			compNumRows2 = compNumRows;
			if ( k == cols-1 ) {
				compNumRows2 = 0;
			}
			k += cols;
		}

		
		//Create the final matrix with the adecuate number of rows and cols
		matrix aux ( numRows , numCols ); 

		int n = 0;
		int m = 0;
		int r2 = 0;
		//Run auxMatrixs and to put this values in the final matrix
		for ( int k = 0 ; k < auxMatrixs.size () ; k++ ) {
			for ( int i = 0 ; i < auxMatrixs[k]-> rows () ; i++ ) {
				for ( int j = 0 ; j < auxMatrixs[k]-> cols () ; j++ ) {
					//Write the values
					aux ( i + n , j + m ) = auxMatrixs[k]-> get_matrix () ( i , j ); 
				}
			}

			//Row change
			if ( k == ( cols-1 ) + r2 ) {
				n = n+auxMatrixs[k]->rows (); 
				m = 0;
				r2 += cols;
			}else
				m = m + auxMatrixs[k]-> cols (); 

		}

		mat = aux;
		system = NULL;
		* this = atomize ( * this ); 
	}catch ( exception &  p ) {
  		outError ( "ERR - Unspected error building Matrix" ); 
	}catch ( int i ) {
  		outError ( "ERR - Incompatible dimensions in input Matrixs" ); 
	}
}


/* 
Constructor with number of rows , cols and one list for compose a new Matrix 
*/
Matrix::Matrix ( long rows , long cols , lst expressions_list ) {
	try{
		if (  ( rows * cols ) != expressions_list.nops () ) throw 1;
			init ( "" , matrix ( rows , cols , expressions_list ) ); 
	}catch ( int e ) {
		outError ( "ERR - Incompatible dimensions in input Matrixs" ); 
	}catch ( exception &  p ) {
		outError ( "ERR - Unspected error building Matrix" ); 
	}
}


/* 
Constructor with name and one list for compose a new Matrix 
*/
Matrix::Matrix ( string name , lst expressions_list ) {
	try{
		//The number of colums is the number of elements of the first sub-list
		matrix aux ( expressions_list.nops () , expressions_list.op ( 0 ).nops () ); 

		//One sub-list is one row in the final Matrix
		for ( int i = 0 ; i < aux.rows () ; i++ ) 
			for ( int j = 0 ; j < aux.cols () ; j++ ) 
				aux ( i , j ) = expressions_list.op ( i ).op ( j ); 
		init ( name , aux ); 
	}catch ( int e ) {
		outError ( "ERR - Incompatible dimensions in input Matrixs" ); 
	}catch ( exception &  p ) {
  		outError ( "ERR - Unspected error building Matrix" ); 
	}
}


/* 
Constructor with one list for compose a new Matrix 
*/
Matrix::Matrix ( lst expressions_list ) {
	try{
		//The number of columns of this Matrix we obtain it for the number of elements of the first list 
		matrix aux ( expressions_list.nops () , expressions_list.op ( 0 ) .nops () ); 

		//One lst is one row of the final Matrix
		for ( int i = 0 ; i< aux.rows () ; i++ ) 
			for ( int j = 0 ; j < aux.cols () ; j++ ) 
				aux ( i , j ) = expressions_list.op ( i ) .op ( j ); 
		init ( "" , aux ); 
	}catch ( int e ) {
		outError ( "ERR - Incompatible dimensions in input Matrixs" ); 
	}catch ( exception & p ) {
  		outError ( "ERR - Unspected error building Matrix" ); 
	}
}


/* 
Return the name of this Matrix
*/
string Matrix::get_name ( void ) {
	return name;
}


/* 
Put the name of this Matrix
*/
void Matrix::set_name ( string new_name ) {
	name = new_name;
}


/* 
Return the matrix of this Matrix
*/
matrix Matrix::get_matrix ( void ) {
	return mat;
}


/* 
Put the matrix of this Matrix
*/
void Matrix::set_matrix ( matrix new_mat ) {
	mat = new_mat;
}


/* 
Destructor method of Matrix
*/
Matrix::~Matrix ( void ) {}


/* 
Operator that return the value of one concrete possition or ERR if this possition don't exist
*/
ex outIndex = symbol ( "ERR" ); 
ex&  Matrix::operator () ( long row , long col ) {
	try{
		//Return the expression of this index fil col
   		return mat ( row , col ); 
	}catch ( exception & p ) {
		outError ( "Index out of range" ); 
		return outIndex;
	}
}


/* 
Return the number of rows
*/
long Matrix::rows ( void ) {
	return mat.rows (); 
}


/* 
Return the number of columns
*/
long Matrix::cols ( void ) {
	return mat.cols (); 
}


/* 
Derivate of this Matrix respect the time "t"
*/
Matrix Matrix::Dt ( void ) {
	try{
		if ( system == NULL ) throw 1;
		matrix aux ( rows () , cols () ); 
		for ( int i = 0 ; i < rows () ; i++ ) 
			for ( int j = 0 ; j < cols () ;j++ ) 
				aux ( i , j ) = system-> dt ( mat ( i , j ) ); 
		Matrix MatrixA ( "d" + name , aux ); 
		return MatrixA;
	}catch ( int e ) {
		outError ( "Invalid System" ); 
		return * this;
	}	
}


/* 
Method that substitute the expresions indicates in the relational form.
*/
Matrix Matrix::subs ( relational relation ) {
	Matrix sol ( rows () , cols () ); 
	Matrix aux = unatomize ( * this ); 
	ex le = relation.lhs () ; 
	ex rig = relation.rhs () ; 
	for ( int i = 0 ; i < rows () ; i++ ) 
		for ( int j = 0 ; j < cols () ; j++ ) {
			sol ( i , j ) = aux ( i , j ).subs ( le + wild ( 1 ) == rig + wild ( 1 ) ); 
			sol ( i , j ) = aux ( i , j ).subs ( le == rig ); 
		}
	return atomize ( sol ); 
}


/* 
Put one System in this Matrix
*/
void Matrix::set_System ( System * new_system ) {
	system = new_system;
}


/* 
Return the System of this Matrix
*/
System * Matrix::get_System ( void ) {
	return system;
}


/* 
Return the Matrix transpose
*/
Matrix Matrix::transpose ( void ) {
	matrix aux = mat.transpose () ; 
	Matrix MatrixA ( ( name + "'" ) , aux ); 
	return MatrixA;
}


/* 
Method that make the Matrix operations calculums
*/
Matrix Matrix::Operations ( const Matrix & MatrixA , const Matrix & MatrixB , int flag ) {

	try{		
		Matrix sol;
		if ( flag == 1 ) {//+
			Matrix result ( MatrixA.mat.add ( MatrixB.mat ) ); 
			sol = result;

		}
		else if ( flag == 2 ) {//* 
			//We don't use .mul () method because we need atomize all the operations
			Matrix result ( MatrixA.mat.rows () , MatrixB.mat.cols () ); 
			ex suma;
			for ( int i = 0 ; i < MatrixA.mat.rows () ; i++ ) {
				for ( int j = 0 ; j < MatrixB.mat.cols () ; j++ ) {		
					suma = 0;
					for ( int z = 0 ; z < MatrixB.mat.rows () ; z++ ) {	
						suma = atomize_ex ( suma + atomize_ex ( MatrixA.mat ( i , z ) * MatrixB.mat ( z , j ) ) ); 
					}
					result.mat ( i , j ) = suma;
				}
			}
			sol = result;
		}
		else if ( flag == 3 ) {//-
			Matrix result ( MatrixA.mat.sub ( MatrixB.mat ) ); 
			sol = result;
		}
		else if ( flag == 4 ) {//- unary
			matrix aux ( MatrixA.mat.rows () , MatrixA.mat.cols () ); 
			Matrix result ( aux.sub ( MatrixA.mat ) ); 
			sol = result;
		}

		return sol;
	}catch ( exception & p ) {
		outError ( "ERR - Failure in Matrix operation" ); 
		Matrix empty;
		return empty;
	}
}


/* 
Method that make the the operations calculates between Matrix and expressions
*/
Matrix Matrix::Operations ( const Matrix & MatrixA , const ex & expression , int flag ) {
	try{
		Matrix sol;
		if ( flag == 1 ) {//expression * Matrix
			matrix aux ( MatrixA.mat.rows () , MatrixA.mat.cols () ); 
			for ( int i = 0; i < aux.rows () ;  i++ ) 
    				for ( int j = 0; j < aux.cols () ;  j++ ) 
          				aux ( i , j ) = expression * MatrixA.mat ( i , j ); 
			Matrix result ( MatrixA.name , aux ); 
			sol = result;
		}
		else if ( flag == 2 ) {//Matrix * expression
			matrix aux ( MatrixA.mat.rows () , MatrixA.mat.cols () ); 
			for ( int i = 0; i < aux.rows () ;  i++ ) 
    				for ( int j = 0; j < aux.cols () ;  j++ ) 
          				aux ( i , j ) = MatrixA.mat ( i , j ) * expression;
			Matrix result ( MatrixA.name , aux ); 
			sol = result;
		}
		return sol;
	}catch ( exception & p ) {
		outError ( "ERR - Failure in Matrix operation" ); 
		Matrix empty;
		return empty;
	}
}


/* 
Sum between two Matrix
*/
Matrix operator + ( const Matrix & MatrixA , const Matrix & MatrixB ) {
	return Matrix::Operations ( MatrixA , MatrixB , 1 ); 
}


/* 
Multiplication between one ex and one Matrix
*/
Matrix operator * ( const ex & expression , const Matrix & MatrixA ) {
	return Matrix::Operations ( MatrixA , expression , 1 ); 
}


/* 
Multiplication between one Matrix and one ex
*/
Matrix operator * ( const Matrix & MatrixA , const ex & expression ) {
	return Matrix::Operations ( MatrixA , expression , 2 ); 
}


/* 
Multiplication between two Matrix
*/
Matrix operator * ( const Matrix & MatrixA , const Matrix & MatrixB ) {
	return Matrix::Operations ( MatrixA , MatrixB , 2 ); 
}


/* 
Diference between two Matrix
*/
Matrix operator - ( const Matrix & MatrixA , const Matrix & MatrixB ) {
	return Matrix::Operations ( MatrixA , MatrixB , 3 ); 
}


/* 
Negation of one Matrix
*/
Matrix operator - ( const Matrix & MatrixA ) {
	return Matrix::Operations ( MatrixA , MatrixA , 4 ); 
}

/*
Operator that help in the inicialized of Matrixes values
*/
Matrix& Matrix::operator , ( const ex & expression ){

	if (last_row > mat.rows()-1 ) return *this;

	this-> mat ( last_row , last_col) = expression;
	
	if (last_col == mat.cols()-1){
		last_row++;
		last_col = 0;
	}
	else
		last_col++;

	return *this;
}


/*
Operator that help in the inicialized of Matrixes values
*/
Matrix& Matrix::operator = ( const ex & expression ){
	last_row = 0;
	last_col = 0;

	this-> mat ( 0 , 0) = expression;

	//If the Matrix oly have one column
	if (( mat.cols() == 1 ) && (mat.rows() >= 1)) 
		last_row++ ;
	else
		last_col++;
	
	return * this;
}

/* 
Matrix output
*/
ostream &  operator << ( ostream &  os , const Matrix & MatrixA ) 
{
	try{
		if ( MatrixA.name == "ERROR" ) throw 1;
		os << MatrixA.name << "\n[";

		matrix aux = MatrixA.mat ; 
		for ( int i = 0; i < aux.rows () ;  i++ ) {
    			for ( int j = 0; j < aux.cols () ;  j++ ) {
          			os << aux ( i , j ); 
				if ( j < aux.cols () -1 ) os << " , ";
				else 
					if (  ( j == aux.cols () -1 ) &&  (  ( i == aux.rows () -1 ) ) ) os << "";
					else os << ";\n";
			}
    		}
		os << "]";
	return os;
	}catch ( exception & p ) {
  		outError ( "ERR - Unespected error in out in Matrix" ); 
		return os;
	}
	catch ( int e ) {
		return os;
	}
}


