#include "System.h"
#include "Globals.h"
#include <fstream>
#include <sstream>

using GiNaC::ex;
using GiNaC::matrix;
using GiNaC::symbol;
using GiNaC::lst;
using GiNaC::numeric;
using GiNaC::csrc_double;
using GiNaC::ex_to;
using GiNaC::mul;
using GiNaC::add;
using GiNaC::is_a;
using std::string;
using std::exception;
using std::vector;
using std::ofstream;
using std::endl;
using std::ostringstream;


/*
External objets from Globals
*/
int gravity;
void ( * outError )( char * );
vector < symbol > atoms;
vector < ex > atom_expressions;
vector < symbol > exclude_atoms;


/*
System's general constructor
*/
void System::init ( void ( * func )( char * ) ) {
	/*Create the "xyz" Base and push back this in the Bases vector*/
	Base * baux = new ( Base );
	symbol x ( "x" );
	Matrix AuxMatrix ( 3 , 1 );
	Base bxyz ( "xyz" , NULL , AuxMatrix , x );
	bxyz.set_System ( this );
	* baux = bxyz;
	Bases.push_back ( baux );//Default Base
		
	/*Create the "O" Point and push back this in the Points vector*/
	Point * paux = new ( Point );
	Vector3D vaux ( baux );
	Point p ( "O" , NULL , & vaux );
	* paux = p; 
	Points.push_back ( paux );//Default Point
	Frame * faux = new ( Frame );
	Frame fabs ( "abs" , paux , baux );
	* faux = fabs;
	Frames.push_back ( faux );//Default Frame
	t.set_name ( "t" );

	outError = func;

}


/*
System's general constructor
*/
System::System ( void ) {
	init ( & printError );
}


/*
Constructor and error function
*/
System::System ( void ( * func )( char * ) ) {
	init ( func );
}


/*
Return de time value
*/
numeric System::get_Time_Value ( void ) {
	return t_numeric;
}


/*
Put the time value
*/
void System::set_Time_Value ( numeric time_value ) {
	t_numeric = time_value;
}


/*
Create a new coordinate, velocity and acceleration in the system and the values of the coordinate, velocity and acceleration
*/
symbol * System::new_Coordinate ( symbol * coordinate , symbol * velocity , symbol * acceleration , numeric coordinate_value , numeric velocity_value , numeric acceleration_value ) {
	coordinates.push_back ( coordinate );
	velocities.push_back ( velocity );
	accelerations.push_back ( acceleration );

	coordinates_numeric.push_back ( coordinate_value );
	velocities_numeric.push_back ( velocity_value );
	accelerations_numeric.push_back ( acceleration_value );
	return coordinate;
}


/*
Create a new coordinate, velocity and acceleration in the system and the values of the coordinate, velocity and acceleration
*/
symbol * System::new_Coordinate ( string coordinate_name , string velocity_name , string acceleration_name , numeric coordinate_value , numeric velocity_value , numeric acceleration_value ) {
	symbol * coordinate = new ( symbol );
	symbol * velocity = new ( symbol );
	symbol * acceleration = new ( symbol );

	symbol coordinate_aux ( coordinate_name );
	* coordinate = coordinate_aux;
	symbol velocity_aux ( velocity_name );
	* velocity = velocity_aux;
	symbol acceleration_aux ( acceleration_name );
	* acceleration = acceleration_aux;

	return new_Coordinate ( coordinate , velocity , acceleration , coordinate_value , velocity_value , acceleration_value );
}


/*
Create a new coordinate, velocity and acceleration in the system and the values of the coordinate, velocity and acceleration
The name of the velocity and the acceleration is auto-generated
*/
symbol * System::new_Coordinate ( string coordinate_name , numeric coordinate_value , numeric velocity_value , numeric acceleration_value ) {
	string velocity_name = "d" + coordinate_name;
	string acceleration_name = "dd" + coordinate_name;

	return new_Coordinate ( coordinate_name , velocity_name , acceleration_name , coordinate_value , velocity_value , acceleration_value );
}


/*
Method for introduce new coordinates , velocities and accelerations in the System
*/
symbol * System::new_Coordinate ( symbol * coordinate , symbol * velocity , symbol * acceleration ) {
	return new_Coordinate ( coordinate , velocity , acceleration , 0.0 , 0.0 , 0.0 );
}


/*
Method for introduce new coordinates , velocities and accelerations in the System, and the coordinate values
*/
symbol * System::new_Coordinate ( symbol * coordinate , symbol * velocity , symbol * acceleration , numeric coordinate_value ) {
	return new_Coordinate ( coordinate , velocity , acceleration , coordinate_value , 0.0 , 0.0 );
}


/*
Method for introduce new coordinates , velocities and accelerations in the System, and the coordinate and velocity values
*/
symbol * System::new_Coordinate ( symbol * coordinate , symbol * velocity , symbol * acceleration , numeric coordinate_value , numeric velocity_value ) {
	return new_Coordinate ( coordinate , velocity , acceleration , coordinate_value , velocity_value , 0.0 );
}


/*
Method for introduce new coordinates , velocities and accelerations in these correspondent vectors
Only to introduce the name of the corrdenate; The velocity and the acceleration ara atomatically generated
*/
symbol * System::new_Coordinate ( string coordinate_name ) {
	return new_Coordinate ( coordinate_name , 0.0 , 0.0 , 0.0 );
}


/*
Method for introduce new coordinates , velocities and accelerations in these correspondent vectors
Only to introduce the name of the corrdenate and the value; The velocity and the acceleration ara atomatically generated
*/
symbol * System::new_Coordinate ( string coordinate_name , numeric coordinate_value ) {
	return new_Coordinate ( coordinate_name , coordinate_value , 0.0 , 0.0 );
}


/*
Method for introduce new coordinates , velocities and accelerations in these correspondent vectors
Only to introduce the name of the corrdenate and the value; The velocity and the acceleration ara atomatically generated
*/
symbol * System::new_Coordinate ( string coordinate_name , numeric coordinate_value , numeric velocity_value ) {
	return new_Coordinate ( coordinate_name , coordinate_value , velocity_value , 0.0 );
}


/*
Method for introduce new parameters in the System and these values
*/
symbol * System::new_Parameter ( symbol * parameter , numeric parameter_value ) {
	parameters.push_back ( parameter );
	parameters_numeric.push_back ( parameter_value );
	
	return parameter;
}


/*
Method for introduce new parameters in the System
*/
symbol * System::new_Parameter ( symbol * parameter ) {
	return new_Parameter ( parameter , 0.0 );
}


/*
Method for introduce new parameters in the System
*/
symbol * System::new_Parameter ( string parameter , numeric parameter_value ) {
	symbol * paux = new ( symbol );
	symbol param ( parameter );
	* paux = param;

	return new_Parameter ( paux , parameter_value );
}


/*
Method for introduce new parameters in the System
*/
symbol * System::new_Parameter ( string parameter ) {
	return new_Parameter ( parameter , 0.0 );
}



/*
Method for introduce new unknown parameters in the System and these values
*/
symbol * System::new_Joint_Unknown ( symbol * unknown_parameter , numeric unknown_parameter_value ) {
	unknowns.push_back ( unknown_parameter );
	unknowns_numeric.push_back ( unknown_parameter_value );
	
	return unknown_parameter;
}


/*
Method for introduce new uknown parameters in the System and these values
*/
symbol * System::new_Joint_Unknown ( string unknown_parameter_name , numeric unknown_parameter_value ) {
	symbol * uaux = new ( symbol );
	symbol param ( unknown_parameter_name );
	* uaux = param;

	return new_Joint_Unknown ( uaux , unknown_parameter_value );
}


/*
Method for introduce new unknown parameters in the System
*/
symbol * System::new_Joint_Unknown ( string unknown_parameter ) {
	return new_Joint_Unknown ( unknown_parameter , 0.0 );
}


/*
Method for introduce new unknown parameters in the System
*/
symbol * System::new_Joint_Unknown ( symbol * unknown_parameter ) {
	return new_Joint_Unknown ( unknown_parameter , 0.0 );
}


/*
Function that return the search symbol in the System in one vector
*/
template <class T>
T Search_Object ( vector < T> vect , string name ) {
	//This function return a pointer
	int i = 0;
	bool encontrado = false;
	while ( ( i < vect.size () ) and ( encontrado == 0 ) )
		if ( name != vect[i]-> get_name () )
			i++;
		else encontrado = true;
	if ( encontrado == true )
		return vect[i];
	else
		return NULL;
}


/*
Method for introduce one Base in the System
*/
void System::new_Base ( Base * BaseA ) {
	try{
		if ( Search_Object ( Bases , BaseA-> get_name () ) != NULL )throw 1;
		Bases.push_back ( BaseA );
	}catch ( exception & p ) {
		outError ( "ERR -  Base not build" );
	}catch ( int e ) {
		outError ( "ERR -  Name of Base already used" );
	}
}


/*
Method for build a new Base in the System
*/
Base * System::new_Base ( string name , Base * previous_base , Matrix rotation_tupla , ex rotation_angle ) {
	try{
		if ( Search_Object ( Bases , name ) != NULL )throw 1;
		Base * baux = new ( Base );
		Base aux ( name , get_Base ( previous_base-> get_name () ) , rotation_tupla , rotation_angle );
		aux.set_System ( this );
		* baux = aux;
		Bases.push_back ( baux );
		return baux;
	}catch ( exception & p ) {
		outError ( "ERR -  Base not build" );
	}catch ( int e ) {
		outError ( "ERR -  Name of Base already used" );
	}
}


/*
Method for build a new Base in the System
*/
Base * System::new_Base ( string name , string previous_base_name , Matrix rotation_tupla , ex rotation_angle ) {
	return new_Base ( name , get_Base ( previous_base_name ) , rotation_tupla , rotation_angle );
}


/*
Method for build a new Base in the System
*/
Base * System::new_Base ( string name , string previous_base_name , ex expression1 , ex expression2 , ex expression3 , ex rotation_angle ) {
	Matrix m_new (  3 , 1 , lst ( expression1 , expression2 , expression3 ) );
	return new_Base ( name , get_Base ( previous_base_name ) , m_new , rotation_angle );
}


/*
Method for build a new Vector3D in the System
*/
Vector3D * System::new_Vector3D ( string name , Matrix mat , Base * base ) {
	try{
		if ( Search_Object ( Vectors , name ) != NULL )throw 1;
		Vector3D * vaux = new ( Vector3D );
		Vector3D aux ( name , mat , base , this );
		* vaux = aux;
		Vectors.push_back ( vaux );
		return vaux;
	}catch ( exception & p ) {
		outError ( "ERR -  Vector3D not build" );
	}catch ( int e ) {
		outError ( "ERR -  Name of Vector3D already used" );
	}
}


/*
Method for build a new Vector3D in the System
*/
Vector3D * System::new_Vector3D ( string name , Matrix mat , string previous_base_name ) {
	return new_Vector3D ( name , mat , get_Base ( previous_base_name ) );
}


/*
Method for build a new Vector3D in the System
*/
Vector3D * System::new_Vector3D ( string name , ex expression1 , ex expression2 , ex expression3 , Base * base ) {
	return new_Vector3D ( name , Matrix ( 3 , 1 , lst ( expression1 , expression2 , expression3 ) ) , base );
}


/*
Method for build a new Vector3D in the System
*/
Vector3D * System::new_Vector3D ( string name , ex expression1 , ex expression2 , ex expression3 , string base_name ) {
		return new_Vector3D ( name , Matrix ( 3 , 1 , lst ( expression1 , expression2 , expression3 ) ) , get_Base ( base_name ) );
}


/*
Method for build a new Vector3D in the System
*/
Vector3D * System::new_Vector3D ( string name , Matrix * mat , string base_name ) {
	return new_Vector3D ( name , * mat , get_Base ( base_name ) );
}


/*
Method for build a new Tensor3D in the System
*/
Tensor3D * System::new_Tensor3D ( string name , Matrix * mat , Base * base ) {
	try{
		if ( Search_Object ( Tensors , name ) != NULL )throw 1;
		Tensor3D * taux = new ( Tensor3D );		
		Tensor3D aux ( name , mat , base );
		aux.set_System ( this );
		* taux = aux;
		Tensors.push_back ( taux );
		return taux;
	}catch ( exception & p ) {
		outError ( "ERR -  Tensor3D not build" );
	}catch ( int e ) {
		outError ( "ERR -  Name of Tensor3D already used" );
	}
}


/*
Method for build a new Tensor3D in the System
*/
Tensor3D * System::new_Tensor3D ( string name , ex exp1 , ex exp2 , ex exp3 , ex exp4 , ex exp5 , ex exp6 , ex exp7 , ex exp8 , ex exp9 , Base * base ) {
	Matrix m ( 3 , 3 , lst ( exp1 , exp2 , exp3 , exp4 , exp5 , exp6 , exp7 , exp8 , exp9 ) );	
	return new_Tensor3D ( name , & m , base );
}


/*
Method for build a new Tensor3D in the System
*/
Tensor3D * System::new_Tensor3D ( string name , ex exp1 , ex exp2 , ex exp3 , ex exp4 , ex exp5 , ex exp6 , ex exp7 , ex exp8 , ex exp9 , string s_Base ) {
	Matrix m ( 3 , 3 , lst ( exp1 , exp2 , exp3 , exp4 , exp5 , exp6 , exp7 , exp8 , exp9 ) );	
	return new_Tensor3D ( name , & m , get_Base ( s_Base ) );
}


/*
Method for build a new Point in the System
*/
Point * System::new_Point ( string name , Point * p , Vector3D * v ) {
	try{
		if ( Search_Object ( Points , name ) != NULL )throw 1;
		Point * paux = new ( Point );		
		Point aux ( name , p , v );
		* paux = aux;
		Points.push_back ( paux );
		return paux;
	}catch ( exception & p ) {
		outError ( "ERR -  Point not build" );
	}catch ( int e ) {
		outError ( "ERR -  Name of Point already used" );
	}
}


/*
Method for build a new Point in the System
*/
Point * System::new_Point ( string name , string p , Vector3D * v ) {
	return new_Point ( name , get_Point ( p ) , v );
}


/*
Method for build a new Frame in the System
*/
Frame * System::new_Frame ( string name , Point * p , Base * base ) {
	try{
		if ( Search_Object ( Frames , name ) != NULL )throw 1;
		Frame * faux = new ( Frame );		
		Frame aux ( name , p , base );
		* faux = aux;
		Frames.push_back ( faux );
		return faux;
	}catch ( exception & p ) {
		outError ( "ERR -  Frame not build" );
	}catch ( int e ) {
		outError ( "ERR -  Name of Frame already used" );
	}
}


/*
Method for build a new Frame in the System
*/
Frame * System::new_Frame ( string name , string s_Point , string s_Base ) {
	return new_Frame ( name , get_Point ( s_Point ) , get_Base ( s_Base ) );
}


/*
Method for build a new Matrix in the System
*/
Matrix * System::new_Matrix ( string name , Matrix m_new ) {
	try{
		if ( Search_Object ( Matrixs , name ) != NULL ) throw 1;
		Matrix * maux = new ( Matrix );
		m_new.set_name ( name );
		m_new.set_System ( this );
		* maux = m_new;
		Matrixs.push_back ( maux );
		return maux;
	}catch ( exception & p ) {
		outError ( "ERR -  Matrix not build" );
	}catch ( int e ) {
		outError ( "ERR -  Name of Matrix already used" );
	}
}


/*
Method for build a new Matrix in the System
*/
Matrix * System::new_Matrix ( Matrix * m ) {
	Matrixs.push_back ( m );
}


/*
Method for assign the symbol corresponding at time
*/
void System::set_Time_Symbol ( symbol time ) {
	t = time;
}


/*
Method for return the symbol time
*/
symbol System::get_Time_Symbol ( void ) {
	return t;
}

	
/*
Return the coordinates's vector
*/
vector < symbol * > System::get_Coordinates ( void ) {
	return coordinates;
}


/*
Return the velocities's vector
*/
vector < symbol * > System::get_Velocities ( void ) {
	return velocities;
}


/*
Return the accelerations's vector
*/
vector < symbol * > System::get_Accelerations ( void ) {
	return accelerations;
}


/*
Return the parameters's vector
*/
vector < symbol * > System::get_Parameters ( void ) {
	return parameters;
}

	
/*
Return the unknown parameters's vector
*/
vector < symbol * > System::get_Unknowns ( void ) {
	return unknowns;
}

	
/*
Return the Bases's vector
*/
vector < Base * > System::get_Bases ( void ) {
	return Bases;
}


/*
Return the Matrix's vector
*/
vector < Matrix * > System::get_Matrixs ( void ) {
	return Matrixs;
}


/*
Return the Frame's vector
*/
vector < Frame * > System::get_Frames ( void ) {
	return Frames;
}


/*
Return the Vector3D's vector
*/
vector < Vector3D * > System::get_Vectors ( void ) {
	return Vectors;
}


/*
Return the Tensor3D's vector
*/
vector < Tensor3D * > System::get_Tensors ( void ) {
	return Tensors;
}


/*
Return the Point's vector
*/
vector < Point * > System::get_Points ( void ) {
	return Points;
}


/*
Return the pointer of one coordinate by name
*/
symbol * System::get_Coordinate ( string name ) {
	try {
		symbol * q = Search_Object ( coordinates , name );
		if ( q == NULL) throw 1;
		else return q;
	} catch (int e) {
		outError ( "ERR - The coordinate donot exist" ); 
		return NULL;
	}
}


/*
Return the pointer of one velocity by name
*/
symbol * System::get_Velocity ( string name ) {
	try {
		symbol * dq = Search_Object ( velocities , name );
		if ( dq == NULL) throw 1;
		else return dq;
	} catch (int e) {
		outError ( "ERR - The velocity donot exist" ); 
		return NULL;
	}
}


/*
Return the pointer of one acceleration by name
*/
symbol * System::get_Acceleration ( string name ) {
	try {
		symbol * ddq = Search_Object ( accelerations , name );
		if ( ddq == NULL) throw 1;
		else return ddq;
	} catch (int e) {
		outError ( "ERR - The acceleration donot exist" ); 
		return NULL;
	}
}


/*
Return the pointer of one parameter by name
*/
symbol * System::get_Parameter ( string name ) {
	try {
		symbol * p = Search_Object ( parameters , name );
		if ( p == NULL) throw 1;
		else return p;
	} catch (int e) {
		outError ( "ERR - The parameter donot exist" ); 
		return NULL;
	}
}


/*
Return the pointer of one parameter by name
*/
symbol * System::get_Unknown ( string name ) {
	try {
		symbol * u = Search_Object ( unknowns , name );
		if ( u == NULL) throw 1;
		else return u;
	} catch (int e) {
		outError ( "ERR - The unknown parameter donot exist" ); 
		return NULL;
	}	
}


/*
Return the pointer of one Base by name
*/
Base * System::get_Base ( string name ) {
	try {
		Base * b = Search_Object ( Bases , name );
		if ( b == NULL) throw 1;
		else return b;
	} catch (int e) {
		return NULL;
	}
}


/*
Return the pointer of one Frame by name
*/
Frame * System::get_Frame ( string name ) {
	try {
		Frame * f = Search_Object ( Frames , name );
		if ( f == NULL) throw 1;
		else return f;
	} catch (int e) {
		outError ( "ERR - The Frame donot exist" ); 
		return NULL;
	}
}


/*
Return the pointer of one Vector3D by name
*/;
Vector3D * System::get_Vector3D ( string name ) {
	try {
		Vector3D * v = Search_Object ( Vectors , name );
		if ( v == NULL) throw 1;
		else return v;
	} catch (int e) {
		outError ( "ERR - The Vector3D donot exist" ); 
		return NULL;
	}	
}


/*
Return the pointer of one Matrix by name
*/
Matrix * System::get_Matrix ( string name ) {
	try {
		Matrix * m = Search_Object ( Matrixs , name );
		if ( m == NULL) throw 1;
		else return m;
	} catch (int e) {
		outError ( "ERR - The Matrix donot exist" ); 
		return NULL;
	}
}


/*
Return the pointer of one Point by name
*/
Point * System::get_Point ( string name ) {
	try {
		Point * p = Search_Object ( Points , name );
		if ( p == NULL) throw 1;
		else return p;
	} catch (int e) {
		outError ( "ERR - The Point donot exist" ); 
		return NULL;
	}
}


/*
Method that it said us if one Base is more near that other of the Base xyz
*/
int System::Bases_Position ( Base * BaseA , Base * BaseB ) {
	Base * bauxA = BaseA;
	int find = 0;
	while ( ( BaseA-> get_name () != "xyz" ) && ( find == 0 ) ) {
		BaseA = get_Base ( BaseA-> get_Previous_Base ()-> get_name () );
		if ( BaseA == BaseB )find = 1;
	}
	BaseA = bauxA;
	return find;//If return 0 , BaseA is more near of Base xyz than BaseB, else return 1
}


/*
Return the pointer of one Base result of reduce two Bases ( these common base in the tree )
*/
Base * System::Reduced_Base ( Base * BaseA , Base * BaseB ) {
	Base * bauxA = BaseA;
	Base * bauxB = BaseB;

	vector < Base * > ramaA;
	vector < Base * > ramaB;

	ramaA.push_back ( BaseA );
	ramaB.push_back ( BaseB );

	while ( BaseA-> get_name() != "xyz" ) {
		ramaA.push_back ( get_Base ( BaseA-> get_Previous_Base ()->get_name () ) );
		BaseA = get_Base ( BaseA-> get_Previous_Base ()-> get_name () );
	}
	while ( BaseB-> get_name () != "xyz" ){
		ramaB.push_back ( BaseB-> get_Previous_Base () );
		BaseB = get_Base ( BaseB-> get_Previous_Base ()-> get_name() );
	}

	int i;
	vector < Base * > ramaAaux;
	vector < Base * > ramaBaux;
	for ( i = ramaA.size () - 1 ; i >= 0 ; i-- ){
		ramaAaux.push_back ( ramaA.back () );
		ramaA.pop_back ();
	}

	for ( i = ramaB.size () - 1 ; i >= 0 ; i-- ){
		ramaBaux.push_back ( ramaB.back () );
		ramaB.pop_back ();
	}		

	//Now run this vectors in pairs and stop when these pairs are'n equals
	i = 0;
	int encontrado = 0;
	while (( i < ramaAaux.size () ) and ( i < ramaBaux.size () ) and ( encontrado == 0 ) )
		if ( ramaAaux[i]-> get_name () == ramaBaux[i]-> get_name () )
			i++;
		else encontrado=1;

	Base * b = ramaAaux[i-1];

	BaseA = bauxA;
	BaseB = bauxB;

	//The Bases stays in the same arm
	if ( ( b == BaseA ) || ( b == BaseB ) ) {
		if ( Bases_Position ( BaseA , BaseB ) == 0 ) {
			if ( gravity == DOWN ){ return BaseA;}
			else if ( gravity == UP ) {return BaseB;}
		}else{
			if ( gravity == DOWN ) {return BaseB;}
			else if ( gravity == UP ) {return BaseA;}
		}
	}else{
		return b;
	}
}


/*
Return the pointer of one Base result of reduce two Bases ( these common base in the tree )
*/
Base * System::Reduced_Base ( string BaseA_name , string BaseB_name ) {
	return Reduced_Base ( get_Base ( BaseA_name ) , get_Base ( BaseB_name ) );
}


/*
Method that it said us if one Point is more near that other of the Point O
*/
int System::Points_Position ( Point * PointA , Point * PointB ) {
	Point * pauxA = PointA;
	int find = 0;
	while ( ( PointA-> get_name () != "O" ) && ( find == 0 ) ) {
		PointA = get_Point ( PointA-> get_Previous_Point ()-> get_name () );
		if ( PointA == PointB ) find = 1;
	}
	PointA = pauxA;
	return find;//If return 0 , PointA is more near of Point O than PointB else return 1
}


/*
Return the pointer of one Point result of reduce two Points ( these common base in the tree )
*/
Point * System::Reduced_Point ( Point * PointA , Point * PointB ) {
	Point * pauxA = PointA;
	Point * pauxB = PointB;

	vector < Point * > ramaA;
	vector < Point * >ramaB;

	ramaA.push_back ( PointA );
	ramaB.push_back ( PointB );

	while ( PointA-> get_name () != "O" ) {
		ramaA.push_back ( get_Point ( PointA-> get_Previous_Point ()-> get_name () ) );
		PointA = get_Point ( PointA-> get_Previous_Point ()-> get_name () );
	}

	while ( PointB-> get_name () != "O" ) {
		ramaB.push_back ( PointB-> get_Previous_Point () );
		PointB = get_Point ( PointB-> get_Previous_Point ()-> get_name () );
	}

	int i;
	vector < Point * > ramaAaux;
	vector < Point * > ramaBaux;
	for ( i = ramaA.size () -1 ; i >= 0 ; i-- ) {
		ramaAaux.push_back ( ramaA.back () );
		ramaA.pop_back () ;
	}

	for ( i = ramaB.size () -1 ; i >= 0 ; i-- ) {
		ramaBaux.push_back ( ramaB.back () );
		ramaB.pop_back () ;
	}		

	//Now run this vectors in pairs and stop when these pairs are'n equals
	i = 0;
	int encontrado = 0;
	while ( ( i < ramaAaux.size () ) and ( i < ramaBaux.size () ) and ( encontrado == 0 ) )
		if ( ramaAaux[i]-> get_name () == ramaBaux[i]-> get_name () )
			i++;
		else encontrado = 1;

	PointA = pauxA;
	PointB = pauxB;

	Point * p = ramaAaux [i-1];	

	//The Points stays in the same arm
	if ( ( p == PointA ) || ( p == PointB ) ) {
		if ( Points_Position ( PointA , PointB ) == 0 ) {
			if ( gravity == DOWN ){ return PointA;}
			else if ( gravity == UP ) {return PointB;}
		}else{
			if ( gravity == DOWN ) {return PointB;}
			else if( gravity == UP ) {return PointA;}
		}
	}else{
		return p;
	}
}



/*
Return the pointer of one Point result of reduce two Points ( these common base in the tree )
*/
Point * System::Reduced_Point ( string PointA_name , string PointB_name ) {
	return Reduced_Point ( get_Point ( PointA_name ) , get_Point ( PointB_name ) );
}


/*
Auxiliar method fot calculate Rotation_Matrix ( BaseA is Reduced_Base )
*/
Matrix System::Rotation_Matrix_Aux ( Base * BaseA , Base * BaseB ) {
	Matrix rt = BaseB-> rotation_matrix () ;
	Matrix bAux;
	while ( BaseB-> get_Previous_Base ()-> get_name () != BaseA-> get_name () ) {
		BaseB = BaseB-> get_Previous_Base () ;
		bAux = BaseB-> rotation_matrix () ;
		rt = rt * bAux;
	}
	return rt;
}


/*
Return the Rotation Matrix between two Bases ( BaseA --> BaseB )
*/
Matrix System::Rotation_Matrix ( Base * BaseA , Base * BaseB ) {
	if ( BaseA != BaseB ) {
		Base * reducedbase = NULL;

		//The gravity no affect here

		if ( gravity == UP ) {
			gravity = DOWN;
			reducedbase = Reduced_Base ( BaseA , BaseB );
			gravity = UP;
		}else
			reducedbase = Reduced_Base ( BaseA , BaseB );


		if ( reducedbase == BaseA )
			return Rotation_Matrix_Aux ( BaseA , BaseB );
		else
			if ( reducedbase == BaseB )
				return Rotation_Matrix_Aux ( BaseB , BaseA ).transpose () ;
			else
				return Rotation_Matrix_Aux ( reducedbase , BaseB ) * Rotation_Matrix_Aux ( reducedbase , BaseA ).transpose () ;
	}else{
		Matrix Aux ( 3 , 3 , lst ( 1 , 0 , 0 , 0 , 1 , 0 , 0 , 0 , 1 ) );
		return Aux;
	}
}


/*
Auxiliar method fot calculate Position_Vector ( Point is reducedbase )
*/
Vector3D System::Position_Vector_Aux ( Point * PointA , Point * PointB ) {

	Vector3D pv = * PointB-> get_Position_Vector ();
	Vector3D vAux;
	while ( PointB-> get_Previous_Point () != PointA ) {
		PointB = PointB-> get_Previous_Point ();
		vAux = * PointB-> get_Position_Vector ();
		pv = pv + vAux;
	}
	return pv;
}


/*
Return the Position Vector between two Points ( PointA --> PointB )
*/
Vector3D System::Position_Vector ( Point * PointA , Point * PointB ) {
	if ( PointA != PointB ) {
		Point * reducedpoint = NULL;

		//The gravity no affect here
		if ( gravity == UP ) {
			gravity = DOWN;
			reducedpoint = Reduced_Point ( PointA , PointB );
			gravity = UP;
		}else
			reducedpoint = Reduced_Point ( PointA , PointB );

		if ( reducedpoint == PointA ) {
			if ( gravity == UP )
				return Position_Vector_Aux ( PointA , PointB );
			return Position_Vector_Aux ( PointA , PointB );
		}else 
			if ( reducedpoint == PointB ) {
				if ( gravity == UP )
					return - Position_Vector_Aux ( PointB , PointA );
				return - Position_Vector_Aux ( PointB , PointA );
			}else{
				if ( gravity == UP )
					return Position_Vector_Aux ( reducedpoint , PointB ) - Position_Vector_Aux ( reducedpoint , PointA );
				return Position_Vector_Aux ( reducedpoint , PointB ) - Position_Vector_Aux ( reducedpoint , PointA );
			}
	}else
		return Vector3D( Matrix ( 3 , 1 ) , NULL );
}


/*
Return the Position Vector between two Points ( PointA --> PointB )
*/
Vector3D System::Position_Vector ( string PointA_name , string PointB_name ) {
	return Position_Vector ( get_Point ( PointA_name ) , get_Point ( PointB_name ) );
}


/*
Auxiliar method fot calculate Angular_Velocity (BaseA is Reduced_Base)
*/
Vector3D System::Angular_Velocity_Aux ( Base * BaseA , Base * BaseB ) {
		
	Vector3D  rt = BaseB-> angular_velocity ();
	rt.set_System ( this );
	Vector3D bAux ( ( ex ) 0 , ( ex ) 0 , ( ex ) 0 , BaseB , this );
	while ( BaseB-> get_Previous_Base () != BaseA ) {
		BaseB = BaseB-> get_Previous_Base ();
		bAux = BaseB-> angular_velocity ();
		bAux.set_System ( this );
		rt = rt + bAux;
	}
	rt.set_System ( this );
	return rt;
}


/*
Return the Angular Velocity between two Bases ( BaseA --> BaseB )
*/
Vector3D System::Angular_Velocity ( Base * BaseA , Base * BaseB ) {
	if ( BaseA != BaseB ) {
		Base * reducedbase = NULL;

		//Comprobe that exist the txo Bases
		try{

			if ( ( Search_Object ( Bases , BaseA-> get_name () ) == NULL) || ( Search_Object ( Bases , BaseB-> get_name () ) == NULL) ) throw 1;
		
			//The gravity no affect here
			if ( gravity == UP ) {
				gravity = DOWN;
				reducedbase = Reduced_Base ( BaseA , BaseB );
				gravity = UP;
			}else
				reducedbase = Reduced_Base ( BaseA , BaseB );
	
			if ( reducedbase == BaseA ) {
				if ( gravity == UP )
					return Angular_Velocity_Aux ( BaseA , BaseB );
				return Angular_Velocity_Aux ( BaseA , BaseB );
			}else
				if ( reducedbase == BaseB ) {
					if ( gravity == UP )
						return -Angular_Velocity_Aux ( BaseB , BaseA );
					return -Angular_Velocity_Aux ( BaseB , BaseA );
				}else{
					if ( gravity == UP )
						return Angular_Velocity_Aux ( reducedbase , BaseB )-Angular_Velocity_Aux ( reducedbase , BaseA );
					return Angular_Velocity_Aux ( reducedbase , BaseB )-Angular_Velocity_Aux ( reducedbase , BaseA );
				}
		}catch ( int i ){
			outError ( "ERR - The two Bases don't exist in this System" ); 
		}

	}else
		return Vector3D ( Matrix ( 3 , 1 ) , BaseB , this );

			
}


/*
Return the Angular Velocity between two Bases of frames ( BaseA --> BaseB )
*/
Vector3D System::Angular_Velocity ( string base_frame_nameA , string base_frame_nameB ) {
	try{
		Base * BaseA = get_Base ( base_frame_nameA );
		if ( BaseA == NULL ) {
			Frame * frame = get_Frame ( base_frame_nameA );
		 	BaseA = frame-> get_Base () ;
		}
		
		Base * BaseB = get_Base ( base_frame_nameB );
		if ( BaseB == NULL ) {
			Frame * frame = get_Frame ( base_frame_nameB );
		 	BaseB = frame-> get_Base () ;
		}
		if ( BaseA == NULL or BaseB == NULL )throw 1;
		return Angular_Velocity_Aux ( BaseA , BaseB );
	}catch ( int i ) {
		Vector3D empty;
		return empty;
	}
}


/*
Return the Angular Velocity tensor between two Bases ( BaseA --> BaseB )
*/
Tensor3D System::Angular_Velocity_Tensor ( Base * BaseA , Base * BaseB ) {
	try{
		if ( get_Base ( BaseA-> get_name () ) == NULL or get_Base ( BaseB-> get_name () ) == NULL )throw 1;
		Vector3D vaux = Angular_Velocity ( BaseA , BaseB );
		Matrix maux ( 3 , 3 );

		maux ( 0 , 1 ) = -vaux ( 2 , 0 );
		maux ( 0 , 2 ) = vaux ( 1 , 0 );
		maux ( 1 , 0 ) = vaux ( 2 , 0 );
		maux ( 1 , 2 ) = -vaux ( 0 , 0 );
		maux ( 2 , 0 ) = -vaux ( 1 , 0 );
		maux ( 2 , 1 ) = vaux ( 0 , 0 );

		Tensor3D sol ( "" , maux , vaux.get_Base () , this );
		return sol;
	}catch ( int i ) {
		Tensor3D empty;
		return empty;
	}
}


/*
Return the Angular Acceleration tensor between two Bases ( BaseA --> BaseB )
*/
Vector3D System::Angular_Acceleration ( string  base_frame_nameA , string  base_frame_nameB ){
	return Dt ( Angular_Velocity ( base_frame_nameA , base_frame_nameB ) ,  base_frame_nameB );
}

/*
Return if the derivate of one ex is zero
*/
bool System::is_dt_zero ( ex expression) {

	expression = unatomize_ex ( expression );
	bool is_zero = true;

	if ( expression.has ( get_Time_Symbol () ) == true ) 
		return is_zero;

	for ( int i = 0; i < get_Coordinates ().size () ; i++ ) 
		if ( 	( expression.has ( * get_Coordinates ()[i] ) == true ) ||
			( expression.has ( * get_Accelerations ()[i] ) == true ) ||
			( expression.has ( * get_Velocities ()[i] ) == true ) ) {
			is_zero=false;
			break;
		}

	return is_zero;
}


/*
Return the ex derivate
*/
ex System::dt ( ex expression ) {

	expression = unatomize_ex ( expression );

	ex expression_derivative = atomize_ex ( diff ( expression , get_Time_Symbol () ) );
	for ( int i = 0; i < get_Coordinates ().size () ; i++ )
		expression_derivative = atomize_ex ( expression_derivative + 
			atomize_ex ( atomize_ex ( atomize_ex ( diff ( expression , ( symbol )* ( get_Coordinates ()[i] ) ) ) * ( symbol )* ( get_Velocities ()[i] ) ) + 
			atomize_ex ( atomize_ex ( diff ( expression , ( symbol ) * ( get_Velocities ()[i] ) ) )* ( symbol ) * ( get_Accelerations ()[i] ) ) ) );
	return expression_derivative ;

}


/*
Return the derivate of one Matrix ( derive all these ex )
*/
Matrix System::Dt ( Matrix MatrixA ) {
	return MatrixA.Dt () ;
}


/*
Return the derivate of one Vector3D respect one Base ( derive all these ex )
*/
Vector3D System::Dt ( Vector3D Vector3DA , Base * base ) {
	return Vector3DA.Dt ( base );
}


/*
Return the derivate of one Vector3D respect one Frame ( derive all these ex )
*/
Vector3D System::Dt ( Vector3D Vector3DA , Frame * frame ) {
	return Vector3DA.Dt ( frame );
}


/*
Return the derivate of one Vector3D respect one Frame or Base ( derive all these ex )
*/
Vector3D System::Dt ( Vector3D Vector3DA , string base_frame_name ) {
	try{
		Base * base = get_Base ( base_frame_name );
		if ( base == NULL ) {
			Frame * frame = get_Frame ( base_frame_name );
			if ( frame == NULL )throw 1;
			else return Vector3DA.Dt ( frame );
		}else
			return Vector3DA.Dt ( base );
	}catch ( int i ) {
		outError ( "ERR - Inexsistent Base or Frame" );
		Vector3D empty;
		return empty;
	}
}


/*
Remove one Matrix of the System by name
*/
void System::remove_Matrix ( string matrix_name ) {
	vector < Matrix * >::iterator it; 
	for ( it = Matrixs.begin () ; it != Matrixs.end () ; it++ )
		if ( ( * it )-> get_name () == matrix_name )
			Matrixs.erase ( it );
}


/*
This method check if the erase of one Vector 3D can be critical because it is referenced by other objects
*/
int System::can_Erase_Vector3D ( string vector3D_name ) {
	int i = 0;
	int encontrado = 0;
	//See in Points vector
	while ( ( i < Points.size () ) and ( encontrado == 0 ) )
		if ( vector3D_name != Points[i]-> get_Position_Vector ()-> get_name () )
			i++;
		else encontrado = 1;
	if ( encontrado == 1 )return -1;
	
	return 1;
}


/*
Remove one Vector3D of the System by name
*/
void System::remove_Vector3D ( string vector3D_name ) {
	try{
		if ( can_Erase_Vector3D ( vector3D_name ) == -1 )throw 1;
		vector < Vector3D * >::iterator it;
		for ( it = Vectors.begin () ; it != Vectors.end () ; it++ )
			if ( ( * it )-> get_name () == vector3D_name )
				Vectors.erase ( it );
	}catch ( int e ) {
		outError ( "ERR - Impossible erase Vector3D" );
	}
}


/*
This method check if the erase of one Point can be critical because it is referenced by other objects
*/
int System::can_Erase_Point ( string point_name ) {
	int i = 0;
	int encontrado = 0;
	if ( Search_Object ( Points , point_name ) == NULL )return -1;

	//See in Frames vector
	while ( ( i < Frames.size () ) and ( encontrado == 0 ) )
		if ( point_name != Frames[i]-> get_Point ()-> get_name () )
			i++;
		else encontrado = 1;
	if ( encontrado == 1 )return -1;
	
	return 1;

}


/*
Remove one Point of the System by name
*/
void System::remove_Point ( string point_name ) {
	try{
		if ( can_Erase_Point ( point_name ) == -1 )throw 1;
		vector < Point * >::iterator it;
		for ( it = Points.begin () ; it != Points.end () ; it++ )
			if ( ( * it )-> get_name () == point_name )
				Points.erase ( it );
	}catch ( int e ) {
		outError ( "ERR - Impossible erase Point" );
	}
}


/*
This method check if the erase of one Base can be critical because it is referenced by other objects
*/
int System::can_Erase_Base ( string base_name ) {
	int i = 0;
	int encontrado = 0;
	//See in Bases vector
	if ( Search_Object ( Bases , base_name ) == NULL )return -1;

	i = 0;
	//See in Vectors vector
	while ( ( i < Vectors.size () ) and ( encontrado == 0 ) )
		if ( base_name != Vectors[i]-> get_Base ()-> get_name () )
			i++;
		else encontrado = 1;
	if ( encontrado == 1 )-1;

	i = 0;
	//See in Tensors vector
	while ( ( i < Tensors.size () ) and ( encontrado == 0 ) )
		if ( base_name != Tensors[i]-> get_Base ()-> get_name () )
			i++;
		else encontrado = 1;
	if ( encontrado == 1 )return -1;

	return 1;
}


/*
Remove one Base of the System by name
*/
void System::remove_Base ( string base_name ) {
	try{
		if ( can_Erase_Point ( base_name ) == -1 )throw 1;
		vector < Base * >::iterator it;
		for ( it = Bases.begin () ; it != Bases.end () ; it++ )
			if ( ( * it )-> get_name () == base_name )
				Bases.erase ( it );
	}catch ( int e ) {
		outError ( "ERR - Impossible erase Base" );
	}
}


/*
Return the jacobian Matrix between two Matrix
*/
Matrix System::jacobian ( Matrix MatrixA , Matrix MatrixB ) {

	Matrix out ( MatrixA.cols () , MatrixB.rows () );
	try{
		if ( ( MatrixA.rows () == 1 ) and ( MatrixB.cols () == 1 ) ) {
			for ( int i = 0 ; i <  MatrixA.cols () ; i++ )
				for ( int j = 0;j< MatrixB.rows () ;j++ )
						out ( i , j ) = unatomize_ex ( MatrixA ( 0 , i ) ).diff ( ex_to<symbol> ( unatomize_ex ( MatrixB ( j , 0 ) ) ) );
		}else
			throw 1;
	}catch ( int e ) {
		outError ( "ERR - Imcompatible dimmension of Matrix" );
	}
	out = atomize ( out );
	return out;
}


/*
Return the jacobian Matrix between one Matrix and one symbol
*/
Matrix System::jacobian ( Matrix MatrixA , symbol symbolA ) {
	Matrix MatrixB ( 1 , 1 );
	MatrixB ( 0 , 0 ) = symbolA; 
	return jacobian ( MatrixA , MatrixB );
}


/*
Return the jacobian Matrix between one Matrix and one ex
*/
Matrix System::jacobian ( ex expression , Matrix MatrixB ) {
	Matrix MatrixA ( 1 , 1 );
	MatrixA ( 0 , 0 ) = expression;
	return jacobian ( MatrixA , MatrixB );
}


/*
Return the derivate between one ex and one symbol
*/
ex System::diff ( ex expression , symbol symbolA ) {
	Matrix MatrixA ( 1 , 1 );
	MatrixA ( 0 , 0 ) = expression;
	Matrix MatrixB = jacobian ( MatrixA , symbolA );
	return MatrixB ( 0 , 0 );
}

/*
Function that make the derivate of ono object ( Matrix ,  Vector3D or Tensor3D ) respect one symbol
*/
template < class T >
T diff ( T object , symbol symbolA ) {
	object = unatomize ( object );
	for ( int i = 0 ; i < object.rows () ; i++ )
		for ( int j = 0 ; j < object.cols() ; j++ )
			object ( i , j ) =atomize_ex ( object ( i , j ).diff ( symbolA ) );
		return object;
}


/*
Return the derivate of one Matrix respect one symbol
*/
Matrix System::diff(Matrix MatrixA,symbol symbolA){
	return ::diff(MatrixA,symbolA);
}

/*
Return the derivate of one Vector3D respect one symbol
*/
Vector3D System::diff(Vector3D VectorA,symbol symbolA){
	return ::diff(VectorA,symbolA);
}


/*
Return the derivate of one Tensor3D respect one symbol
*/
Tensor3D System::diff(Tensor3D TensorA,symbol symbolA){
	return ::diff(TensorA,symbolA);
}


/*
Return one Matrix with all the symbols of one vector
*/
Matrix Matrix_Of_Symbols ( vector < symbol * > vect ) {
	Matrix aux ( vect.size () , 1 );
	for ( int i = 0 ; i <  vect.size () ; i++ ) {
		aux ( i , 0 ) = * vect[i];
	}
	return aux;
}


/*
Return one Matrix with all the coordinates
*/
Matrix System::Coordinates ( void ) {
	return Matrix_Of_Symbols ( coordinates );
}


/*
Return one Matrix with all the accelerations
*/
Matrix System::Accelerations ( void ) {
	return Matrix_Of_Symbols ( accelerations );
}


/*
Return one Matrix with all the velocities
*/
Matrix System::Velocities ( void ) {
	return Matrix_Of_Symbols ( velocities );
}


/*
Return one Matrix with all the unknown symbols
*/
Matrix System::Joint_Unknowns ( void ) {
	return Matrix_Of_Symbols ( unknowns );
}


/*
This method evaluate one expression , substituting the variables from her value
*/
ex System::numeric_evaluate ( ex expression ) {
	try{
		expression = unatomize_ex ( expression );
		for ( int i = 0 ; i < coordinates.size () ; i++ ) {
		        expression = evalf ( expression.subs ( * coordinates[i] == coordinates_numeric[i] ) );
		        expression = evalf ( expression.subs ( * velocities[i] == velocities_numeric[i] ) );
		        expression = evalf ( expression.subs ( * accelerations[i] == accelerations_numeric[i] ) );
		}

		for ( int i = 0 ; i < parameters.size () ; i++ )
		        expression = evalf ( expression.subs ( * parameters[i] == parameters_numeric[i] ) );
	
		for ( int i = 0 ; i < unknowns.size () ; i++ )
		        expression = evalf ( expression.subs ( * unknowns[i] == unknowns_numeric[i] ) );
	}catch ( exception & p ) {
		outError ( "ERR- in values evaluation" );
	}
	return expression;
	
}


/*
Method that print one doble pointer
*/
string System::print_Array ( long rows , long cols , double ** array ) {
	ostringstream os;
	for ( int i = 0 ; i < rows ; i++ ) {
		for ( int j = 0 ; j < cols ; j++ )
			os<< array[i][j] <<" ";
		os<<"\n";
	}
	return os.str () ;
}


/*
Method that evalue all the elements of one Matrix and return other Matrix
*/
Matrix System::evaluate_Matrix ( Matrix MatrixA ) {
	Matrix n ( MatrixA.rows () , MatrixA.cols () );
	try{			
		ex expression;

		for ( int i = 0; i < MatrixA.rows () ; i++ )
			for ( int j = 0; j < MatrixA.cols () ; j++ ) {
				expression = numeric_evaluate ( MatrixA ( i , j ) );
				if ( !is_a<numeric> ( expression ) )throw 1;
				n ( i , j ) = ex_to<numeric> ( expression ).to_double () ;
			}
	}catch ( int i ) {
		outError ( "ERR- in values evaluation" );
	}
	return n;
}


/*
Method that evalue all the elements of one Matrix and return a double pointer of doubles
*/
double ** System::evaluate_Array ( Matrix MatrixA ) {
	double *  * n = new double * [MatrixA.rows ()];
	for ( int i = 0 ; i <  MatrixA.rows () ; i++ ) {
		n[i] = new double[MatrixA.cols ()];
	}
	ex expression;
	
	try{
		for ( int i = 0; i < MatrixA.rows () ; i++ ) {
			for ( int j = 0; j < MatrixA.cols () ; j++ ) {
				expression = numeric_evaluate ( MatrixA ( i , j ) );
				if ( !is_a < numeric > ( expression ) )throw 1;
				n[i][j] = ex_to < numeric > ( expression ).to_double () ;
			}
		}
	}catch ( int i ) {
		outError ( "ERR- in values evaluation" );
	}
	
	return n;
}


/*
Method that create a C file for export parameters
*/
void System::export_var_def_C ( void ) {
	ofstream cout;
	cout.open ( "var_def.c" );
	cout << csrc_double ;
	cout << "//----------var_def.c lib3D_MEC expoted-----------" << endl << endl;
	cout << "//----------time-----------" << endl << endl;
	cout << "double " << get_Time_Symbol () << ";" << endl;
	cout << "//----------parameters-----------" << endl << endl;
	for ( int i = 0 ; i < parameters.size () ; i++ )
		cout << "double " << parameters[i]-> get_name () << ";" << endl;
	cout.close () ;
}


/*
Method that create a header C file for export parameters
*/
void System::export_var_def_H ( void ) {
	ofstream cout;
	cout.open ( "var_def.h" );
	cout << csrc_double ;
	cout << "//----------var_def.h lib3D_MEC exported-----------" << endl << endl;
	cout << "//----------time-----------" << endl << endl;
	cout << "extern double " << get_Time_Symbol () << ";" << endl;
	cout << "//----------parameters-----------" << endl << endl;
	for ( int i = 0 ; i < parameters.size () ; i++ )
		cout << "extern double " << parameters[i]-> get_name () << ";" << endl;
	cout.close ();
}


/*
Method that create a C file for initialize the parameters
*/
void System::export_var_init_C ( void )
{
	ofstream cout;
	cout.open ( "var_init.c" );
	cout << csrc_double ;
	cout << "//----------var_init.c lib3D_MEC expoted-----------" << endl << endl;
	cout << "#include <math.h>" << endl;
	cout << "#include \"var_def.h\"" << endl;
	cout << "void var_init ( void )" << endl;
	cout << "{" << endl;
	cout << "//----------time-----------" << endl << endl;
	cout <<  get_Time_Symbol () << " = " << t_numeric << ";" << endl;
	cout << "//----------parameters-----------" << endl << endl;
	for ( int i = 0 ; i < parameters.size () ; i++ )
		cout  << parameters[i]-> get_name () << " = " << parameters_numeric[i] << ";" << endl;
	cout << "}" << endl;
	cout.close ();
}


/*
Method that create a C file for export atoms
*/
void System::export_atom_def_C ( lst atom_list ) {
	if ( atomization == YES ) {
		ofstream cout;
		cout.open ( "atom_def.c" );
		cout << csrc_double ;
		cout << "//----------atom_def.c lib3D_MEC expoted-----------" << endl << endl;	
		cout << "//----------atoms-----------" << endl << endl;

		for ( size_t i = 0; i < atom_list.nops () ; ++i )
			cout << "double " << atom_list.op ( i )<< ";" << endl;			

		cout.close ();

		cout.open ( "atom_def.h" );
		cout << csrc_double ;
		cout << "//----------atom_def.h lib3D_MEC expoted-----------" << endl << endl;
		cout << "//----------atoms-----------" << endl << endl;

		for ( size_t i = 0; i < atom_list.nops () ; ++i ) 
			cout << "extern double " << atom_list.op ( i ) << ";" << endl;			
	
		cout.close ();
	}
}


/*
Method that create a header C file for export coordinates
*/
void System::export_gen_coord_vect_def_H ( void ){
	ofstream cout;
	cout.open ( "gen_coord_vect_def.h" );
	cout << csrc_double ;
	cout << "//----------gen_coord_vect_def.h lib3D_MEC expoted-----------" << endl << endl;
	cout << "#include <stdio.h>" << endl;
	cout << "#include <math.h>" << endl;
	cout << "#include <gsl/gsl_vector.h>" << endl;
	cout << "extern gsl_vector * q;" << endl;
	cout << "#define n_gen_coord " << coordinates.size () << endl;
	cout << "//----------time-----------" << endl << endl;
	cout << "extern double " << get_Time_Symbol () << ";" << endl;
	cout << "//----------Coordinates-----------" << endl << endl;
	for ( int i = 0 ; i < coordinates.size () ; i++ )
		cout << "extern double " << coordinates[i]-> get_name () << ";" << endl;
	cout.close () ;
}


/*
Method that create a C file for initialize coordinates
*/
void System::export_gen_coord_vect_init_C ( void ) {
	ofstream cout;
	cout.open ( "gen_coord_vect_init.c" );
	cout << csrc_double ;
	cout << "#include <stdio.h>" << endl;
	cout << "#include <math.h>" << endl;
	cout << "#include <gsl/gsl_vector.h>" << endl;
	cout << "#include \"var_def.h\"" << endl;
	cout << "#include \"gen_coord_vect_def.h\"" << endl;
	for ( int i = 0 ; i < coordinates.size () ; i++ )
		cout << "#define " << coordinates[i]-> get_name () << " " << coordinates_numeric[i] << endl;
	cout << "void gen_coord_vect_init ( void )" << endl;
	cout << "{" << endl;
	cout << "  if ( n_gen_coord > 0 )" << endl;
	cout << "  {" << endl;
	cout << "  q = gsl_vector_alloc ( n_gen_coord );" << endl;
	for ( int i = 0 ; i < coordinates.size () ; i++ )
		cout << "gsl_vector_set ( q , " << i << " , " << coordinates[i]-> get_name () << " );" << endl;
	cout << "  }" << endl;
	cout << "}" << endl;
	cout.close () ;
}


/*
Method that create a header C file for export velocities
*/
void System::export_gen_vel_vect_init_H ( void ) {
	ofstream cout;
	cout.open ( "gen_vel_vect_def.h" );
	cout << csrc_double ;
	cout << "//----------gen_vel_vect_def.h lib3D_MEC expoted-----------" << endl << endl;
	cout << "#include <stdio.h>" << endl;
	cout << "#include <math.h>" << endl;
	cout << "#include <gsl/gsl_vector.h>" << endl;
	cout << "extern gsl_vector * dq;" << endl;
	cout << "#define n_gen_vel " << velocities.size () << endl;
	cout << "//----------time-----------" << endl << endl;
	cout << "extern double " << get_Time_Symbol () << ";" << endl;
	cout << "//----------Velocities-----------" << endl << endl;
	for ( int i = 0 ; i < velocities.size () ; i++ )
		cout << "extern double " << velocities[i]-> get_name () << ";" << endl;
	cout.close () ;
}


/*
Method that create a C file for initializa velocities
*/
void System::export_gen_vel_vect_init_C ( void ) {
	ofstream cout;
	cout.open ( "gen_vel_vect_init.c" );
	cout << csrc_double ;
	cout << "#include <stdio.h>" << endl;
	cout << "#include <math.h>" << endl;
	cout << "#include <gsl/gsl_vector.h>" << endl;
	cout << "#include \"var_def.h\"" << endl;
	cout << "#include \"gen_vel_vect_def.h\"" << endl;
	for ( int i = 0 ; i < velocities.size () ; i++ )
		cout << "#define " << velocities[i]-> get_name () << " " << velocities_numeric[i] << endl;
	cout << "void gen_vel_vect_init ( void )" << endl;
	cout << "{" << endl;
	cout << "  if ( n_gen_vel > 0 )" << endl;
	cout << "  {" << endl;
	cout << "  dq = gsl_vector_alloc ( n_gen_vel );" << endl;
	for ( int i = 0 ; i < velocities.size () ; i++ )
		cout << "gsl_vector_set ( dq , " << i << " , " << velocities[i]-> get_name () << " );" << endl;
	cout << "  }" << endl;
	cout << "}" << endl;
	cout.close () ;
}


/*
Method that create a header C file for export accelerations
*/
void System::export_gen_accel_vect_init_H ( void ) {
	ofstream cout;
	cout.open ( "gen_accel_vect_def.h" );
	cout << csrc_double ;
	cout << "//----------gen_accel_vect_def.h lib3D_MEC expoted-----------" << endl << endl;
	cout << "#include <stdio.h>" << endl;
	cout << "#include <math.h>" << endl;
	cout << "#include <gsl/gsl_vector.h>" << endl;
	cout << "extern gsl_vector * ddq;" << endl;
	cout << "#define n_gen_accel " << accelerations.size () << endl;
	cout << "//----------time-----------" << endl << endl;
	cout << "extern double " << get_Time_Symbol () << ";" << endl;
	cout << "//----------Acceleratios-----------" << endl << endl;
	for ( int i = 0 ; i < accelerations.size () ; i++ )
		cout << "extern double " << accelerations[i]-> get_name () << ";" << endl;
	cout.close () ;
}


/*
Method that create a C file for initialize accelerations
*/
void System::export_gen_accel_vect_init_C ( void ) {
	ofstream cout;
	cout.open ( "gen_accel_vect_init.c" );
	cout << csrc_double ;
	cout << "#include <stdio.h>" << endl;
	cout << "#include <math.h>" << endl;
	cout << "#include <gsl/gsl_vector.h>" << endl;
	cout << "#include \"var_def.h\"" << endl;
	cout << "#include \"gen_accel_vect_def.h\"" << endl;
	for ( int i = 0 ; i < accelerations.size () ; i++ )
		cout << "#define " << accelerations[i]-> get_name () << " " << accelerations_numeric[i] << endl;
	cout << "void gen_accel_vect_init ( void )" << endl;
	cout << "{" << endl;
	cout << "  if ( n_gen_accel > 0 )" << endl;
	cout << "  {" << endl;
	cout << "  ddq = gsl_vector_alloc ( n_gen_accel );" << endl;
	for ( int i = 0 ; i < accelerations.size () ; i++ )
		cout << "gsl_vector_set ( ddq , " << i << " , " << accelerations[i]-> get_name () << " );" << endl;
	cout << "  }" << endl;
	cout << "}" << endl;
	cout.close () ;
}


/*
Method that create a header C file for export unknown parameters
*/
void System::export_unknowns_vect_init_H ( void ) {
	ofstream cout;
	cout.open ( "unknowns_vect_def.h" );
	cout << csrc_double ;
	cout << "//----------unknowns_vect_def.h lib3D_MEC expoted-----------" << endl << endl;
	cout << "#include <stdio.h>" << endl;
	cout << "#include <math.h>" << endl;
	cout << "#include <gsl/gsl_vector.h>" << endl;
	cout << "extern gsl_vector * unknowns;" << endl;
	cout << "#define n_unknowns " << unknowns.size () << endl;
	cout << "//----------time-----------" << endl << endl;
	cout << "extern double " << get_Time_Symbol () << ";" << endl;
	cout << "//----------Unknowns-----------" << endl << endl;
	for ( int i = 0 ; i < unknowns.size () ; i++ )
		cout << "extern double " << unknowns[i]-> get_name () << ";" << endl;
	cout.close () ;
}


/*
Method that create a C file for initialize unknowns parameters
*/
void System::export_unknowns_vect_init_C ( void ) {
	ofstream cout;
	cout.open ( "unknowns_vect_init.c" );
	cout << csrc_double ;
	cout << "#include <stdio.h>" << endl;
	cout << "#include <gsl/gsl_vector.h>" << endl;
	cout << "#include \"var_def.h\"" << endl;
	cout << "#include \"unknowns_vect_def.h\"" << endl;
	cout << "#include <math.h>" << endl;
	for ( int i = 0 ; i < unknowns.size () ; i++ )
		cout << "#define " << unknowns[i]-> get_name () << " " << unknowns_numeric[i] << endl;
	cout << "void unknowns_vect_init ( void )" << endl;
	cout << "{" << endl;
	cout << "  if ( n_unknowns > 0 )" << endl;
	cout << "  {" << endl;
	cout << "  unknowns = gsl_vector_alloc ( n_unknowns );" << endl;
	cout << "  gsl_vector_set_zero ( unknowns );" << endl;
	cout << "  }" << endl;
	cout << "}" << endl;
	cout.close () ;
}

/*
Method that create a C file
*/
void System::export_Column_Matrix_C ( string function_name , string vector_name , Matrix Col_Matrix , lst Col_Matrix_atom_list ) {
	ofstream cout;
	cout.open ( ( function_name+".c" ).c_str () );
	cout << csrc_double;
	cout << "//----------"<< function_name <<".c lib3D_MEC expoted-----------" << endl << endl;
	cout << "//For increased velocity use GSL HAVE_INLINE and GSL_RANGE_CHECK_OFF preprocesor directive to compile:" << endl << endl;
	cout << "// gcc -DHAVE_INLINE = 1 -DGSL_RANGE_CHECK_OFF = 1 ... ---" << endl << endl;
	cout << "#include <stdio.h>" << endl;
	cout << "#include <gsl/gsl_vector.h>" << endl;
	cout << "#include \"var_def.h\"" << endl;
	cout << "#include \"gen_coord_vect_def.h\"" << endl;
	cout << "#include \"gen_vel_vect_def.h\"" << endl;
	cout << "#include \"gen_accel_vect_def.h\"" << endl;
	cout << "#include \"unknowns_vect_def.h\"" << endl;
	cout << "#include <math.h>" << endl;

	for ( int i = 0 ; i < coordinates.size () ; i++ )
		cout << "#define " << coordinates[i]-> get_name () << " gsl_vector_get ( q , " << i << " )" << endl;
	for ( int i = 0 ; i < velocities.size () ; i++ )
		cout << "#define " << velocities[i]-> get_name () << " gsl_vector_get ( dq , " << i << " )" << endl;
	for ( int i = 0 ; i < accelerations.size () ; i++ )
		cout << "#define " << accelerations[i]-> get_name () << " gsl_vector_get ( ddq , " << i << " )" << endl;
	for ( int i = 0 ; i < unknowns.size () ; i++ )
		cout << "#define " << unknowns[i]-> get_name () << " gsl_vector_get ( unknowns , " << i << " )" << endl;

	if ( atomization == YES ) {
		lst aux_Col_Matrix_atom_list; 
		aux_Col_Matrix_atom_list = Col_Matrix_atom_list; 
		atom_list ( Col_Matrix , aux_Col_Matrix_atom_list );
		for ( int i = 0 ; i < aux_Col_Matrix_atom_list.nops () ; ++i )
			cout << "extern double " << aux_Col_Matrix_atom_list.op ( i )<< ";" << endl;
	}

	if ( 0 )
		cout << "gsl_vector * " << function_name << " ( gsl_vector * " << vector_name << " )" << endl;
	else{
		cout << "gsl_vector * " << vector_name << " = NULL;" << endl;
		cout << "gsl_vector * " << function_name << " ()" << endl;
	}
	cout << "{" << endl;
	cout << "size_t n_comp = " << Col_Matrix.rows () << ";" << endl;
	cout << "if ( " << vector_name << " == NULL )" << endl;
	cout << " {" << endl;
	cout << "    " << vector_name << " = gsl_vector_alloc ( n_comp );" << endl;

	vector < int > vector_aux_one;
	vector < int > vector_aux_two;

	lst Col_Matrix_atom_expression_list; 
	atom_expression_list ( Col_Matrix_atom_list , Col_Matrix_atom_expression_list );

	if ( atomization == YES )
		for ( int i = 0 ; i < Col_Matrix_atom_list.nops () ; i++ )
			if ( is_dt_zero ( Col_Matrix_atom_list.op ( i )) == true )
				cout << "/*constant*/ " << Col_Matrix_atom_list.op ( i ) << " = " << Col_Matrix_atom_expression_list.op ( i )<< ";" << endl;
			else 
				vector_aux_one.push_back ( i );

	for ( int i = 0 ; i < Col_Matrix.rows () ; i++ )
		if ( is_dt_zero ( Col_Matrix ( i , 0 ) ) == true )
			cout << "gsl_vector_set ( " << vector_name << " , " << i <<" , " << Col_Matrix ( i , 0 )<< " );" << endl;
		else
			vector_aux_two.push_back ( i );
		
	cout << " }" << endl;


	if ( atomization == YES )
		for ( int i = 0 ; i < vector_aux_one.size () ; i++ )
			cout << " " << Col_Matrix_atom_list.op ( vector_aux_one.at ( i ) ) << " = " << Col_Matrix_atom_expression_list.op ( vector_aux_one.at ( i ) )<< ";" << endl; 		

	for ( int i = 0 ; i < vector_aux_two.size () ; i++ )
		cout << "gsl_vector_set ( " << vector_name << " , " << vector_aux_two.at ( i ) <<" , " << Col_Matrix ( vector_aux_two.at ( i ) , 0 )<< " );" << endl;

	cout << "return " << vector_name << ";" << endl;
	cout << "}" << endl;
	cout.close () ;
	cout.open ( ( function_name+".h" ).c_str () );
	if ( 0 )
	cout << "extern gsl_vector * " << function_name << " ( gsl_vector * " << vector_name << " );" << endl;
	else {
		cout << "extern 	gsl_vector * " << vector_name << ";" << endl;
		cout << "extern		gsl_vector * " << function_name << " () ;" << endl;
	}
	cout.close () ;
}


/*
Method that create a C file
*/
void System::export_Matrix_C ( string function_name , string matrix_name , Matrix Matrix , lst Matrix_atom_list ) {
	ofstream cout;
	cout.open ( ( function_name+".c" ).c_str () );
	cout << csrc_double ;
	cout << "//----------"<< function_name <<".c lib3D_MEC expoted-----------" << endl << endl;
	cout << "//For increased velocity use GSL HAVE_INLINE and GSL_RANGE_CHECK_OFF preprocesor directive to compile:" << endl << endl;
	cout << "// gcc -DHAVE_INLINE = 1 -DGSL_RANGE_CHECK_OFF = 1 ... ---" << endl << endl;
	cout << "#include <stdio.h>" << endl;
	cout << "#include <gsl/gsl_matrix.h>" << endl;
	cout << "#include \"var_def.h\"" << endl;
	cout << "#include \"gen_coord_vect_def.h\"" << endl;
	cout << "#include \"gen_vel_vect_def.h\"" << endl;
	cout << "#include \"gen_accel_vect_def.h\"" << endl;
	cout << "#include \"unknowns_vect_def.h\"" << endl;
	cout << "#include <math.h>" << endl;
	for ( int i = 0 ; i < coordinates.size () ; i++ )
		cout << "#define " << coordinates[i]-> get_name () << " gsl_vector_get ( q , " << i << " )" << endl;
	for ( int i = 0 ; i < velocities.size () ; i++ )
		cout << "#define " << velocities[i]-> get_name () << " gsl_vector_get ( dq , " << i << " )" << endl;
	for ( int i = 0 ; i < accelerations.size () ; i++ )
		cout << "#define " << accelerations[i]-> get_name () << " gsl_vector_get ( ddq , " << i << " )" << endl;
	for ( int i = 0 ; i < unknowns.size () ; i++ )
		cout << "#define " << unknowns[i]-> get_name () << " gsl_vector_get ( unknowns , " << i << " )" << endl;

	if ( atomization == YES ) {
		lst aux_Matrix_atom_list; 
		aux_Matrix_atom_list = Matrix_atom_list; 
		atom_list ( Matrix , aux_Matrix_atom_list );
		for ( int i = 0; i < aux_Matrix_atom_list.nops () ; ++i )
			cout << "extern double " << aux_Matrix_atom_list.op ( i )<< ";" << endl;
	}

	if ( 0 ) 
		cout << "gsl_matrix * " << function_name << " ( gsl_matrix * " << matrix_name << " )" << endl;
	else {
		cout << "gsl_matrix * " << matrix_name << " = NULL;" << endl;
		cout << "gsl_matrix * " << function_name << " ()" << endl;
	}

	cout << "{" << endl;
	cout << "size_t n_rows = " << Matrix.rows () << ";" << endl;
	cout << "size_t n_cols = " << Matrix.cols () << ";" << endl;
	cout << "if ( " << matrix_name << " == NULL )" << endl;
	cout << " {" << endl;
	cout << "    " << matrix_name << " = gsl_matrix_alloc ( n_rows , n_cols );" << endl;
	cout << "    gsl_matrix_set_zero ( " << matrix_name << " );" << endl;

	
	vector < int > vector_aux_one;
	vector < int > vector_aux_two;
	vector < int > vector_aux_three;

	lst Matrix_atom_expression_list; 
	atom_expression_list ( Matrix_atom_list , Matrix_atom_expression_list );

	if ( atomization == YES )
		for ( int i = 0 ; i < Matrix_atom_list.nops () ; i++ )
			if ( is_dt_zero ( Matrix_atom_list.op ( i ) ) == true )
				cout << "/*constant*/ " << Matrix_atom_list.op ( i ) << " = " << Matrix_atom_expression_list.op ( i )<< ";" << endl;
			else 
				vector_aux_one.push_back ( i );

	for ( int i = 0 ; i < Matrix.rows () ; i++ )
		for ( int j = 0 ; j < Matrix.cols () ; j++ )
			if ( is_dt_zero ( Matrix ( i , j ) ) == true )
				cout << "gsl_matrix_set ( " << matrix_name << " , " << i <<" , " << j << " , " << Matrix ( i , j )<< " );" << endl;
			else{
				vector_aux_two.push_back ( i );
				vector_aux_three.push_back ( j );
			}

	cout << " }" << endl;


	if ( atomization == YES )
		for ( int i = 0 ; i < vector_aux_one.size () ; i++ )
			cout << " " << Matrix_atom_list.op ( vector_aux_one.at ( i ) ) << " = " << Matrix_atom_expression_list.op ( vector_aux_one.at ( i ) )<< ";" << endl; 		


	for ( int i = 0 ; i < vector_aux_two.size () ; i++ )
			cout << "gsl_matrix_set ( " << matrix_name << " , " << vector_aux_two.at ( i ) <<" , " << vector_aux_three.at ( i ) << " , " << Matrix ( vector_aux_two.at ( i ) , vector_aux_three.at ( i ) )<< " );" << endl;


	cout << "return " << matrix_name << ";" << endl;
	cout << "}" << endl;
	cout.close () ;
	cout.open ( ( function_name+".h" ).c_str () );
	if ( 0 )
		cout << "extern gsl_matrix * " << function_name << " ( gsl_matrix * " << matrix_name << " );" << endl;
	else {
		cout << "extern 	gsl_matrix * " << matrix_name << ";" << endl;
		cout << "extern		gsl_matrix * " << function_name << " () ;" << endl;
	}
	cout.close () ;
}


/*
Method that create a C file
*/
void System::export_write_state_file_header_C ( void ) {
	ofstream cout;
	cout.open ( "write_state_file_header.c" );
	cout << csrc_double ;
	cout << "//----------write_state_file_header.c lib3D_MEC expoted-----------" << endl << endl;
	cout << "#include <stdio.h>" << endl;
	cout << "#include <math.h>" << endl;
	cout << "#include \"var_def.h\"" << endl;
	cout << "void write_state_file_header ( FILE * state_file )" << endl;
	cout << "{" << endl;
	cout << "fprintf ( state_file , \"";
	cout << get_Time_Symbol () << "\t" ;
	for ( int i = 0 ; i < coordinates.size () ; i++ )
		cout << coordinates[i]-> get_name () << "\t" ;
	for ( int i = 0 ; i < velocities.size () ; i++ )
		cout << velocities[i]-> get_name () << "\t" ;
	for ( int i = 0 ; i < accelerations.size () ; i++ )
		cout << accelerations[i]-> get_name () << "\t" ;
	for ( int i = 0 ; i < parameters.size () ; i++ )
		cout << parameters[i]-> get_name () << "\t" ;
	for ( int i = 0 ; i < unknowns.size () ; i++ )
		cout << unknowns[i]-> get_name () << "\t" ;
	cout << "\\n\" );" << endl;
	cout << "}" << endl;
	cout.close () ;
}

/*
Method that create a C file
*/
void System::export_write_state_file_C ( void ) {
	ofstream cout;
	cout.open ( "write_state_file.c" );
	cout << csrc_double ;
	cout << "//----------write_state_file.c lib3D_MEC expoted-----------" << endl << endl;
	cout << "#include <stdio.h>" << endl;
	cout << "#include \"var_def.h\"" << endl;
	cout << "#include \"gen_coord_vect_def.h\"" << endl;
	cout << "#include \"gen_vel_vect_def.h\"" << endl;
	cout << "#include \"gen_accel_vect_def.h\"" << endl;
	cout << "#include \"unknowns_vect_def.h\"" << endl;
	cout << "#include <math.h>" << endl;
	for ( int i = 0 ; i < coordinates.size () ; i++ )
		cout << "#define " << coordinates[i]-> get_name () << " gsl_vector_get ( q , " << i << " )" << endl;
	for ( int i = 0 ; i < velocities.size () ; i++ )
		cout << "#define " << velocities[i]-> get_name () << " gsl_vector_get ( dq , " << i << " )" << endl;
	for ( int i = 0 ; i < accelerations.size () ; i++ )
		cout << "#define " << accelerations[i]-> get_name () << " gsl_vector_get ( ddq , " << i << " )" << endl;
	for ( int i = 0 ; i < unknowns.size () ; i++ )
		cout << "#define " << unknowns[i]-> get_name () << " gsl_vector_get ( unknowns , " << i << " )" << endl;
	cout << "void write_state_file ( FILE * state_file )" << endl;
	cout << "{" << endl;
	cout << "fprintf ( state_file , \"";
	cout << "%g\t";
	for ( int i = 0 ; i < coordinates.size () ; i++ )
		cout << "%g\t";
	for ( int i = 0 ; i < velocities.size () ; i++ )
		cout << "%g\t";
	for ( int i = 0 ; i < accelerations.size () ; i++ )
		cout << "%g\t";
	for ( int i = 0 ; i < parameters.size () ; i++ )
		cout << "%g\t";
	for ( int i = 0 ; i < unknowns.size () ; i++ )
		cout << "%g\t";
	cout << "\\n\"";
	cout << " , " << get_Time_Symbol () ;
	for ( int i = 0 ; i < coordinates.size () ; i++ )
		cout << " , " << coordinates[i]-> get_name () ;
	for ( int i = 0 ; i < velocities.size () ; i++ )
		cout << " , " << velocities[i]-> get_name () ;
	for ( int i = 0 ; i < accelerations.size () ; i++ )
		cout << " , " << accelerations[i]-> get_name () ;
	for ( int i = 0 ; i < parameters.size () ; i++ )
		cout << " , " << parameters[i]-> get_name () ;
	for ( int i = 0 ; i < unknowns.size () ; i++ )
		cout << " , " << unknowns[i]-> get_name () ;
	cout << " );" << endl;
	cout << "}" << endl;
	cout.close () ;
}


/*
System's destructor method
*/
System::~System ( void ) {}



