#ifndef SYSTEM_H
#define SYSTEM_H

#include "Base.h"
#include "Vector3D.h"
#include "Tensor3D.h"
#include "Frame.h"
#include "Point.h"
#include <ginac/ginac.h>

using GiNaC::ex;
using GiNaC::matrix;
using GiNaC::symbol;
using GiNaC::lst;
using GiNaC::numeric;
using std::string;
using std::vector;

	class System{

		//Atributes

		symbol t;
		numeric t_numeric;
    		vector < symbol * > coordinates; //q
    		vector < symbol * > velocities; //dq
    		vector < symbol * > accelerations; //ddq
    		vector < symbol * > parameters;
		vector < symbol * > unknowns;

		vector < numeric > coordinates_numeric;
		vector < numeric > velocities_numeric;
		vector < numeric > accelerations_numeric;
		vector < numeric > parameters_numeric;
		vector < numeric > unknowns_numeric;

		vector < Base * > Bases;
		vector < Matrix * > Matrixs;
		vector < Vector3D * > Vectors;
		vector < Tensor3D * > Tensors;
		vector < Frame * > Frames;
		vector < Point * > Points;
	private:

		//Private methods

		int can_Erase_Point ( string point_name );
		int can_Erase_Base ( string base_name );
		int can_Erase_Vector3D ( string Vector3D_name );
		Vector3D Angular_Velocity_Aux ( Base * BaseA , Base * BaseB );
		Vector3D Position_Vector_Aux ( Point * PointA , Point * PointB );
		Matrix Rotation_Matrix_Aux ( Base * BaseA , Base * BaseB );
		int Bases_Position ( Base * BaseA , Base * BaseB );
		int Points_Position ( Point * PointA , Point * PointB );
		void init ( void  ( * func ) ( char * ) );
  	public:

		//Constructors
		
		System ( void );
		System ( void  ( * func ) ( char * ) );
		
		//Public methods

		symbol * new_Coordinate ( symbol * coord , symbol * vel , symbol * accel , numeric coord_value , numeric vel_value , numeric accel_value );
		symbol * new_Coordinate ( symbol * coord , symbol * vel , symbol * accel , numeric coord_value , numeric vel_value );
		symbol * new_Coordinate ( symbol * coord , symbol * vel , symbol * accel , numeric coord_value );
		symbol * new_Coordinate ( symbol * coord , symbol * vel , symbol * accel );

		symbol * new_Coordinate ( string coord_name , string vel_name , string accel_name , numeric coord_value , numeric vel_value , numeric accel_value );
		symbol * new_Coordinate ( string coord_name , numeric coord_value , numeric vel_value , numeric accel_value );
		symbol * new_Coordinate ( string coord_name , numeric coord_value , numeric vel_value );
		symbol * new_Coordinate ( string coord_name , numeric coord_value );
		symbol * new_Coordinate ( string coord_name );

		symbol * new_Parameter ( symbol * parameter , numeric parameter_value );
		symbol * new_Parameter ( symbol * parameter );
		symbol * new_Parameter ( string parameter_name , numeric parameter_value );
		symbol * new_Parameter ( string parameter_name );

		symbol * new_Joint_Unknown ( string unknown_parameter_name );
		symbol * new_Joint_Unknown ( string unknown_parameter_name , numeric unknown_parameter_value );
		symbol * new_Joint_Unknown ( symbol * unknown_parameter );
		symbol * new_Joint_Unknown ( symbol * unknown_parameter , numeric unknown_parameter_value );

		void new_Base ( Base * BaseA );
		Matrix * new_Matrix ( Matrix * MatrixA );
		void new_Vector3D ( Vector3D * Vector3DA );
		void set_Time_Symbol ( symbol timesymbol );

		Base * new_Base ( string name , Base * previous_base , Matrix rotation_tupla , ex rotation_angle );
		Base * new_Base ( string , string previous_base_name , Matrix rotationmatrix , ex rotation_angle );
		Base * new_Base ( string name , string previous_base_name , ex expression1 , ex expression2 , ex expression3 , ex rotation_angle );
		Vector3D * new_Vector3D ( string name , Matrix mat , Base * base );
		Vector3D * new_Vector3D ( string name , Matrix mat , string base_name );
		Vector3D * new_Vector3D ( string name , Matrix * mat , string base_name );
		Vector3D * new_Vector3D ( string name , ex expression1 , ex expression2 , ex expression3 , Base * base );
		Vector3D * new_Vector3D ( string name , ex expression1 , ex expression2 , ex expresion3 , string base_name );
		Tensor3D * new_Tensor3D ( string name , Matrix * mat , Base * base );
		Tensor3D * new_Tensor3D ( string name , ex exp1 , ex exp2 , ex exp3 , ex exp4 , ex exp5 , ex exp6 , ex exp7 , ex exp8 , ex exp9 , Base * base );
		Tensor3D * new_Tensor3D ( string name , ex exp1 , ex exp2 , ex exp3 , ex exp4 , ex exp5 , ex exp6 , ex exp7 , ex exp8 , ex exp9 , string base_name );
		Point * new_Point ( string name , Point * previous_point , Vector3D * position_vector );
		Point * new_Point ( string name , string previous_point_name , Vector3D * position_vector );
		Frame * new_Frame ( string name , Point * point , Base * base );
		Frame * new_Frame ( string name , string point_name , string base_name );
		Matrix * new_Matrix ( string name , Matrix mat );

		symbol get_Time_Symbol ( void );
		vector < symbol * > get_Coordinates ( void );
		vector < symbol * > get_Velocities ( void );
		vector < symbol * > get_Accelerations ( void );
		vector < symbol * > get_Parameters ( void );
		vector < symbol * > get_Unknowns ( void );
		vector < Base * > get_Bases ( void );
		vector < Matrix * > get_Matrixs ( void );
		vector < Vector3D * > get_Vectors ( void );
		vector < Tensor3D * > get_Tensors ( void );
		vector < Point * > get_Points ( void );
		vector < Frame * > get_Frames ( void );
		
		Matrix Coordinates ( void );
		Matrix Accelerations ( void );
		Matrix Velocities ( void );
		Matrix Joint_Unknowns ( void );

		symbol * get_Coordinate ( string coordinate_name );
		symbol * get_Velocity ( string velocity_name );
		symbol * get_Acceleration ( string acceleration_name );
		symbol * get_Parameter ( string parameter_name );
		symbol * get_Unknown ( string unknown_parameter_name );
 		Base * get_Base ( string base_name );
 		Frame * get_Frame ( string frame_name );
 		Matrix * get_Matrix ( string Matrix_name );
		Vector3D * get_Vector3D ( string vector3D_name );
		Point * get_Point ( string point_name );

		Base * Reduced_Base ( string BaseA_name , string BaseB_name );
		Base * Reduced_Base ( Base * BaseA , Base * BaseB );
		Point * Reduced_Point ( string PointA_name , string PointB_name );
		Point * Reduced_Point ( Point * PointA , Point * PointB );

		Matrix Rotation_Matrix ( Base * BaseA , Base * BaseB );
		Vector3D Position_Vector ( Point * PointA , Point * PointB );
		Vector3D Position_Vector ( string PointA_name , string PointB_name );
		Vector3D Angular_Velocity ( Base * BaseA , Base * BaseB );
		Vector3D Angular_Velocity ( string  base_frame_nameA , string  base_frame_nameB );
		Tensor3D Angular_Velocity_Tensor ( Base * BaseA , Base * BaseB );
		Vector3D Angular_Acceleration ( string  base_frame_nameA , string  base_frame_nameB );
		void remove_Matrix ( string matrix_name );
		void remove_Vector3D ( string vector3D_name );
		void remove_Point ( string point_name );
		void remove_Base ( string base_name );

		ex dt ( ex expression );
		bool is_dt_zero ( ex expression);
		Matrix Dt ( Matrix MatrixA );
		Vector3D Dt ( Vector3D Vector3DA , Base * base );
		Vector3D Dt ( Vector3D Vector3DA , Frame * frame );
		Vector3D Dt ( Vector3D Vector3DA , string base_frame_name );
		
		numeric get_Time_Value ( void );	
		void set_Time_Value ( numeric time_value );

		Matrix jacobian ( Matrix MatrixA , Matrix MatrixB );
		Matrix jacobian ( ex , Matrix MatrixA );
		Matrix jacobian ( Matrix MatrixA , symbol symbolA );
		ex jacobian ( ex expression , symbol symbolA );
		ex diff ( ex expression , symbol symbolA );
		Matrix diff ( Matrix Matrixa , symbol symbolA );
		Vector3D diff ( Vector3D Vector3DA , symbol symbolA );
		Tensor3D diff ( Tensor3D Tensor3DA , symbol symbolA );
		ex numeric_evaluate ( ex expression );
		Matrix evaluate_Matrix ( Matrix MatrixA );
		double *  * evaluate_Array ( Matrix MatrixA );
		string print_Array ( long rows , long cols , double *  * array );
		
		//Export functions

		void export_var_def_C ( void );
		void export_var_def_H ( void );
		void export_var_init_C ( void );
		void export_atom_def_C ( lst atom_list );
		void export_gen_coord_vect_def_H ( void );
		void export_gen_coord_vect_init_C ( void );
		void export_gen_vel_vect_init_H ( void );
		void export_gen_vel_vect_init_C ( void );
		void export_gen_accel_vect_init_H ( void );
		void export_gen_accel_vect_init_C ( void );
		void export_unknowns_vect_init_H ( void );
		void export_unknowns_vect_init_C ( void );
		void export_Column_Matrix_C ( string , string , Matrix , lst );
		void export_Matrix_C ( string , string , Matrix , lst );
		void export_write_state_file_header_C ( void );
		void export_write_state_file_C ( void );

		//Destructor

		~System ( void );
	};

#endif // SYSTEM_H
